function [alphaExt, clExt, cdExt] = extendpolar(method, limit, alpha, cl, cd)
% EXTENDPOLAR Extends polars to the full range of AOA ([-180, 180] deg).
%   This function implements different empirical methods to extend the polars 
%   outside of their usual range.
%
%   This function can either take a full Polar structure (generated with
%   xf2mat) or values for angles of attack and their associated cl and cd.
%
%   If the input is specified as aoa, cl and cd, the three arrays must have the 
%   same number of rows (one for each angle of attack). The columns indicate 
%   values for different polars (different Mach, different Reynolds,...). 
%   For example, if the cl values are given for 2 different Reynols and there
%   are 100 values for the aoa, cl and cd must be [100 x 2].
%
% IMPORTANT:
%   To be extended, the input polar must AT LEAST containt the angle of attack 
%   of 0 deg and the stall angle.
%
% <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox">Documentation (README)</a>
% <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox/-/issues">Report an issue</a>
% -----
%
% Synopsis:
%   [ALPHAEXT, CLEXT, CDEXT] = EXTENDPOLAR prompts the user to input a
%   Polar structure and sets the default method and limit
%
%   [ALPHAEXT, CLEXT, CDEXT] = EXTENDPOLAR(METHOD) prompts the user to
%   input a Polar structure, but uses the defined METHOD. The default LIMIT is
%   used.
%
%   [ALPHAEXT, CLEXT, CDEXT] = EXTENDPOLAR(METHOD, LIMIT) prompts the
%   user to input a Polar structure, but uses the defined METHOD and LIMIT.
%
%   [ALPHAEXT, CLEXT, CDEXT] = EXTENDPOLAR(METHOD, LIMIT, ALPHA, CL, CD)
%   extends the ALPHA, CL vand CD alues to the full range of angles of attack 
%   using the defined METHOD and LIMIT.
%
% Inputs:
%   method : Method for extending the polars ('viterna')
%   limit  : Limit to start the polar extension ('last', 'stall')
%   Polar  : Polar structure
%   alpha  : Vector with angles of attack
%   cl     : Array with lift coefficients [aoas x nbPolar]
%   cd     : Array with drag coefficients [aoas x nbPolar]
%
% Output:
%   alphaExt : Extended range of angles of attack, IN RADIANS
%   clExt    : Extended lift coefficient array
%   cdExt    : Extended drag coefficient array
%
% Example:
%   EXTENDPOLAR
%   EXTENDPOLAR('viterna')
%   EXTENDPOLAR('viterna', 'stall')
%   EXTENDPOLAR('viterna', 'stall', Polar)
%   EXTENDPOLAR('viterna', 'stall', alpha, cl, cd)
%
% See also: XF2MAT, PLOTPOLARS.

% -----
% Copyright 2022 Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
% ------------------------------------------------------------------------------


%% Import other functions from this package
import af_tools.findstall
import af_tools.utils.*


%% Constants and defaults

DEFAULT_METHOD = 'viterna';
DEFAULT_LIMIT = 'last';
ALLOWED_METHODS = {'viterna'};
ALLOWED_LIMITS = {'stall', 'last'};


%% Check inputs

% This function does not work with only 4 inputs
if nargin == 4
    error('MATLAB:extendpolar:wrongNumberOfInputs',['This functions requires '...
        '0, 1, 2, 3 or 5 inputs. You provided 4. Type ''help extendpolar'' '...
        'for details on how to use this function.']);
end


% Check method and limit
if nargin >= 1
    validateattributes(method, {'char','string'},{'scalartext'}, mfilename(), ...
        'method', 1)
    method = validatestring(method, ALLOWED_METHODS, mfilename(), 'method',1);
    
    if nargin >= 2
        validateattributes(limit, {'char','string'},{'scalartext'}, mfilename(), ...
            'spacing', 3)
        limit = validatestring(limit, ALLOWED_LIMITS, mfilename(), 'limit',2);
    else
        limit = DEFAULT_LIMIT;
    end
    
else
    method = DEFAULT_METHOD;
    limit = DEFAULT_LIMIT;
end

% If no arguments, prompt user to input a Polar file
if nargin <=2 
    [filename, path] = uigetfile('*.mat', 'Select a Polar file');
    tmp = load([path,filename]);
    Polar = tmp.Polar;
    clear tmp;
end

if nargin == 3
    % If 3 arguments reassign Polar variable properly
    Polar = alpha;
    clear alpha;
end

% Validate Polar structure
if nargin <= 3
    validateattributes(Polar, {'struct'},{'nonempty'}, mfilename(), 'Polar', 3)
    [alpha, cl, cd] = extractpolar(Polar);
    clear Polar;
end

% Validate alpha input
alpha = vecttocol(alpha);
validateattributes(alpha, {'numeric'},{'nonempty'}, mfilename(), ...
    'alpha')

% Validate cl input
cl = vecttocol(cl);
validateattributes(cl, {'numeric'},{'nrows', size(alpha,1)}, mfilename(),...
    'cl')

% Validate cd input
cd = vecttocol(cd);
validateattributes(cd, {'numeric'},{'nrows', size(alpha,1)}, mfilename(),...
    'cd')

% Standardize alpha size so we have one column per polar as well
if size(alpha,2) == 1
    alpha = repmat(alpha, [1, size(cl,2)]);
end

% Check if polar have enough data points to be extended
if sign(min(alpha(:,1))) == sign(max(alpha(:,1)))
    error('MATLAB:extendpolar:noZero',['To be extended, a polar must at '...
        'least contain the AOA = 0 deg.']);
end
try
    findstall(alpha, cl, cd);
catch
    error('MATLAB:extendpolar:noStall',['To be extended, a polar must at '...
        'least contain the stall point.']);
end


%% Extend polars

% Extend using the selected method
switch method
    case 'viterna'
        for i = 1:size(alpha,2)
            [alphaExt(:,i), clExt(:,i), cdExt(:,i)] = viterna(limit, alpha(:,i), cl(:,i), cd(:,i));
        end   
end

end


function [alphaExt, clExt, cdExt] = viterna(limit, alpha, cl, cd)
% VITERNA Implementation of the original Viterna model
% Source:
%   Viterna & Corrigan. 1982. “Fixed Pitch Rotor Performance of Large Horizontal
%   Axis Wind Turbines”.

% Note:
%   The typical subscript 's' used in most definitions is here replaced by the
%   generic 'l' (for limit) as the function can be used either with the stall
%   point or the last available data.
%
% Process (the number will indicate the index of alpha, cl, cd):
%   1. Extend Cl/Cd to ]limit, 90]
%   2. Extend to ]90, 180-limit]
%   3. Extend to ]180-limit,180] (lift must be corrected by linear interpolation)
%   4. Extend to [-limit, -alpha(1)[
%   5. Extend to [-90,-limit[
%   6. Flip 3 and 4 to get the reminaing part
%   EXTRA. Trim negative drag values to the minimum drag of the original polar


% Import other functions from this package
import af_tools.findstall
import af_tools.utils.*


% Constants
DEFAULT_AR = 10; % AR as a rather small impact. 10 is a good compromise
SCALING_CL = 0.7; % Suggested in AeroDyn's manual

% Trim the polars to nonNaN values only 
% (some Cl, Cd may be NaN if they come from xf2mat with trimAoas = false)
id = find(~isnan(cl) == 1);
alpha = alpha(id);
cl = cl(id);
cd = cd(id);

% Get limit point
if strcmp(limit, 'stall')
    [alpha_l, cl_l, cd_l] = findstall(alpha, cl, cd);
    % Trim polar values after stall point
    ids = find(alpha>alpha_l);
    alpha(ids) = [];
    cl(ids) = [];
    cd(ids) = [];
elseif strcmp(limit, 'last')
    alpha_l = alpha(end);
    cl_l = cl(end,:);
    cd_l = cd(end,:);
end

% Viterna's coefficients
sas = sin(alpha_l);
cas = cos(alpha_l);

cd_max = 1.11 + 0.018 * DEFAULT_AR;
VitCoeffs.a1 = cd_max/2;
VitCoeffs.b1 = cd_max;
VitCoeffs.a2 = (cl_l - cd_max .* sas .*cas) .* sas ./ cas.^2;
VitCoeffs.b2 = (cd_l - cd_max .* sas.^2) ./ cas;

% Minimum step between 2 AOA
alpha_step = (min(diff(alpha)));

% 1. Extend Cl/Cd to ]limit, 90]
alpha1 = (alpha_l+alpha_step : alpha_step : pi/2)';
[cl1, cd1] = viternaeq(VitCoeffs, alpha1, 1);

% 2. Extend to ]90, 180-limit]
alpha2 = (pi/2+alpha_step : alpha_step : pi-alpha_l)';
[cl2, cd2] = viternaeq(VitCoeffs, pi-alpha2, -SCALING_CL);

% 3. Extend to ]180-limit,180]
alpha3 = (pi-alpha_l+alpha_step : alpha_step : pi)';
[~, cd3] = viternaeq(VitCoeffs, pi-alpha3, 1);
% Correct lift with linear variation
cl3 = (alpha3 - pi) / alpha_l * cl_l * SCALING_CL ;

% 4. Extend to [-limit, -alpha(1)[ (by linear interpolation)
if alpha(1) <= -alpha_l
    alpha4 = [];
    cl4 = [];
    cd4 = [];
    alpha_lneg = alpha(1);
else
    alpha4 = (-alpha_l : alpha_step : alpha(1) - alpha_step)';
    cl4 = interp1([-alpha_l,alpha(1)], [-cl_l*SCALING_CL,cl(1)],alpha4);
    cd4 = interp1([-alpha_l,alpha(1)], [cd_l,cd(1)],alpha4);
    alpha_lneg = -alpha_l;
end

% 5. Extend to [-90,-limit[
alpha5 = (-pi/2 : alpha_step : alpha_lneg-alpha_step)';
[cl5, cd5] = viternaeq(VitCoeffs, alpha5, SCALING_CL);

% 6. Remaining part is simply a relfexion
alpha6 = (-pi : alpha_step : -pi/2-alpha_step)';
cl6 = -flipud([cl2;cl3]);
cd6 = flipud([cd2;cd3]);

% Concatenate everything to get the full extended polars
alphaExt = [alpha6; alpha5; alpha4; alpha; alpha1; alpha2; alpha3];
clExt = [cl6; cl5; cl4; cl; cl1; cl2; cl3];
cdExt = [cd6; cd5; cd4; cd; cd1; cd2; cd3];

% Trim negatives cd to cdmin
cdmin = min(cd);
cdExt(cdExt <= cdmin) = cdmin;

end


function [cl, cd] = viternaeq(VitCoeffs, alpha, clScaling)
% VITERNAEQ Viterna's equation for the extension of the polars
% Source:
%   Viterna & Corrigan. 1982. “Fixed Pitch Rotor Performance of Large Horizontal
%   Axis Wind Turbines”.

cl = VitCoeffs.a1 * sin(2*alpha) + VitCoeffs.a2 * cos(alpha).^2./sin(alpha);
cl = cl * clScaling;

cd = VitCoeffs.b1 * sin(alpha).^2 + VitCoeffs.b2  * cos(alpha);

end