function [alpha_s, cl_s, cd_s] = findstall(alpha, cl, cd)
% FINDSTALL Finds the stall point and returns the corresponding AOA, CL and CD.
%   This function can either take a full Polar structure (generated with
%   xf2mat) or values for angles of attack and their associated cl and cd.
%
%   If the input is specified as aoa, cl and cd, the three arrays must have the 
%   same number of rows (one for each angle of attack). The columns indicate 
%   values for different polars (different Mach, different Reynolds,...). 
%   For example, if the cl values are given for 2 different Reynols and there
%   are 100 values for the aoa, cl and cd must be [100 x 2].
%
% <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox">Documentation (README)</a>
% <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox/-/issues">Report an issue</a>
% -----
%
% Synopsis:
%   [ALPHA_S, CL_S, CD_S] = FINDSTALL prompts the user to input a Polar structure.
%
%   [ALPHA_S, CL_S, CD_S] = FINDSTALL(POLAR) finds the stall point and
%   the corresponding CL and CD based on the POLAR structure given in input.
%
%   [ALPHA_S, CL_S] = FINDSTALL(ALPHA, CL) finds the stall point and
%   the corresponding CL based on the polars defined by ALPHA, CL. As no CD is
%   provided, CD_S can not be required for output.
%
%   [ALPHA_S, CL_S, CD_S] = FINDSTALL(ALPHA, CL, CD) finds the stall point and
%   the corresponding CL and CD based on the polars defined by ALPHA, CL, CD.
%
% Inputs:
%   Polar : Polar structure
%   alpha : Vector with angles of attack
%   cl    : Array with lift coefficients [aoas x nbPolar]
%   cd    : Array with drag coefficients [aoas x nbPolar]
%
% Output:
%   alpha_s : Stall angle of attack, IN RADIANS
%   cl_s    : Lift at stall angle of attack
%   cd_s    : Drag at stall angle of attack
%
% Example:
%   FINDSTALL
%   FINDSTALL(Polar)
%   FINDSTALL(alpha, cl)
%   FINDSTALL(alpha, cl, cd)
%
% See also: XF2MAT, FINDZEROLIFT, FINDCLLINEARRANGE.

% -----
% Copyright 2022 Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
% ------------------------------------------------------------------------------


%% Import other functions from this package
import af_tools.utils.*


%% Check inputs

% If no arguments, prompt user to input a Polar file
if nargin == 0
    [filename, path] = uigetfile('*.mat', 'Select a Polar file');
    tmp = load([path,filename]);
    Polar = tmp.Polar;
    clear tmp;
end

if nargin == 1
    % If one single argument reassign variable properly
    Polar = alpha;
    clear alpha;
end

% Validate Polar structure
if nargin <= 1
    validateattributes(Polar, {'struct'},{'nonempty'}, mfilename(), 'Polar', 1)
    [alpha, cl, cd, ~] = extractpolar(Polar);
    clear Polar;
end

% Validate alpha
alpha = vecttocol(alpha);
validateattributes(alpha, {'numeric'},{'nonempty'}, mfilename(), ...
    'alpha')
if size(alpha,1) <= 3
    error('MATLAB:findstall:notEnoughData', ['findstall requires at least 3 '...
        'points on the polars to find the stall angle of attack.']);
end

% Validate cl
cl = vecttocol(cl);
validateattributes(cl, {'numeric'},{'nrows', size(alpha,1)}, mfilename(),...
    'cl')

% Validate cd
if nargin == 2
    nargoutchk(1,2)
else
    cd = vecttocol(cd);
    validateattributes(cd, {'numeric'},{'nrows', size(alpha,1)}, mfilename(),...
        'cd')
end

% Standardize alpha size so we have one column per polar as well
if size(alpha,2) == 1
    alpha = repmat(alpha, [1, size(cl,2)]);
end


%% Find stall for each polar

alpha_s = zeros(1, size(cl,2));
cl_s = zeros(1, size(cl,2));
cd_s = zeros(1, size(cl,2));

if nargin == 2
    for i = 1:size(cl,2)
        [alpha_s(i), cl_s(i)] = findstallvect(alpha, cl(:,i));
    end
    
else
    for i = 1:size(cl,2)
        [alpha_s(i), cl_s(i), cd_s(i)] = findstallvect(alpha(:,i), cl(:,i), cd);
    end
end


end


%% HELPERS

function [alpha_s, cl_s, cd_s] = findstallvect(alpha, cl, cd)
% FINDSTALLVECT Finds the stall point and returns the corresponding AOA, CL and CD
%   For vectors!

PLOT_POLAR_IF_ERROR = true; % Plot the polar if error so user can see why it fails

% Find stall for each polar using findpeaks
maxFound = false;
minWidth = 0;

% Iterate in order to find only one peak (largest one should indicate the stall)
while ~maxFound && minWidth <= length(alpha)
    [~,locs] = findpeaks(cl,'MinPeakWidth',minWidth);
    if numel(locs)==1
        maxFound = true;
    else
        minWidth = minWidth + 1;
    end
end

if ~maxFound
    % If findpeaks does not work
    if PLOT_POLAR_IF_ERROR
        figure('Name', 'Debug: Polar with no stall')
        plot(rad2deg(alpha), cl)
        title('Polar with no clear stall peak')
        xlabel('AOA [deg]')
        ylabel('C_l [-]')
        grid on
    end
    error('MATLAB:findstall:noStallFound', ['findpeaks did not found any ' ...
        'stall point the polar. Please provide a polar that goes a bit after '...
        'the stall.\n' ...
        'See the attached plot for more details']);
end

alpha_s = alpha(locs);
cl_s = cl(locs);
if nargin == 3
    cd_s = cd(locs);
else
    cd_s = NaN;
end

end
