function [alpha_zeroL, cd_zeroL] = findzerolift(alpha, cl, cd)
% FINDZEROLIFT Finds the zero-lift angle and the associate C_d.
%   This function can either take a full Polar structure (generated with
%   xf2mat) or values for angles of attack and their associated cl and cd.
%
%   If the input is specified as aoa, cl and cd, the three arrays must have the 
%   same number of rows (one for each angle of attack). The columns indicate 
%   values for different polars (different Mach, different Reynolds,...). 
%   For example, if the cl values are given for 2 different Reynols and there
%   are 100 values for the aoa, cl and cd must be [100 x 2].
%
% <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox">Documentation (README)</a>
% <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox/-/issues">Report an issue</a>
% -----
%
% Synopsis:
%   [ALPHA_ZEROL, CD_ZEROL] = FINDZEROLIFT prompts the user to input a Polar structure.
%
%   [ALPHA_ZEROL, CD_ZEROL] = FINDZEROLIFT(POLAR) finds the zero-lift angle and
%   the corresponding CD based on the POLAR structure given in input.
%
%   [ALPHA_ZEROL, ] = FINDZEROLIFT(ALPHA, CL) finds the zero-lift angle and
%   the corresponding CD based on the polars defined by ALPHA, CL. As no CD is
%   provided, CD_ZEROL can not be required for output.
%
%   [ALPHA_ZEROL, CD_ZEROL] = FINDZEROLIFT(ALPHA, CL, CD) finds the zero-lift angle and
%   the corresponding CD based on the polars defined by ALPHA, CL, CD.
%
% Inputs:
%   Polar : Polar structure
%   alpha : Vector with angles of attack
%   cl    : Array with lift coefficients [aoas x nbPolar]
%   cd    : Array with drag coefficients [aoas x nbPolar]
%
% Output:
%   alpha_zeroL : Zero-lift angle of attack, IN RADIANS
%   cd_zeroL    : Drag at zero-lift angle of attack
%
% Example:
%   FINDZEROLIFT
%   FINDZEROLIFT(Polar)
%   FINDZEROLIFT(alpha, cl)
%   FINDZEROLIFT(alpha, cl, cd)
%
% See also: XF2MAT, FINDSTALL, FINDCLLINEARRANGE.

% -----
% Copyright 2022 Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
% ------------------------------------------------------------------------------


%% Import other functions from this package
import af_tools.utils.*


%% Check inputs

% If no arguments, prompt user to input a Polar file
if nargin == 0
    [filename, path] = uigetfile('*.mat', 'Select a Polar file');
    tmp = load([path,filename]);
    Polar = tmp.Polar;
    clear tmp;
end

if nargin == 1
    % If one single argument reassign variable properly
    Polar = alpha;
    clear alpha;
end

% Validate Polar structure
if nargin <= 1
    validateattributes(Polar, {'struct'},{'nonempty'}, mfilename(), 'Polar', 1)
    [alpha, cl, cd, ~] = extractpolar(Polar);
    clear Polar;
end

% Validate alpha
alpha = vecttocol(alpha);
validateattributes(alpha, {'numeric'},{'nonempty'}, mfilename(), ...
    'alpha')
if size(alpha,1) <= 3
    error('MATLAB:findzerolift:notEnoughData', ['findzerolift requires at least 3 '...
        'points on the polars to find the zero lift angle of attack.']);
end

% Validate cl
cl = vecttocol(cl);
validateattributes(cl, {'numeric'},{'nrows', size(alpha,1)}, mfilename(),...
    'cl')

% Validate cd
if nargin == 2
    nargoutchk(1,1)
else
    cd = vecttocol(cd);
    validateattributes(cd, {'numeric'},{'nrows', size(alpha,1)}, mfilename(),...
        'cd')
end

% Standardize alpha size so we have one column per polar as well
if size(alpha,2) == 1
    alpha = repmat(alpha, [1, size(cl,2)]);
end


%% Find zero lift for each polar

alpha_zeroL = zeros(1,size(cl,2));
cd_zeroL = zeros(1,size(cl,2));

if nargin == 2
    for i = 1:size(cl,2)
        alpha_zeroL(i) = findzeroliftvect(alpha, cl(:,i));
    end
    
else
    for i = 1:size(cl,2)
        [alpha_zeroL(i), cd_zeroL(i)] = findzeroliftvect(alpha(:,i), cl(:,i), cd(:,i));
    end
end


end


%% HELPERS

function [alpha_zeroL, cd_zeroL] = findzeroliftvect(alpha, cl, cd)
% FINDZEROLIFTVECT Finds the zero-lift point and returns the corresponding AOA and CD
%   For vectors!

% Process:
%   1. Get the value of the last negative and first positive Cl
%   2. Use linear interpolation between the two

PLOT_POLAR_IF_ERROR = true; % Plot the polar if error so user can see why it fails

idx = find(cl(2:end) > 0, 1);  % Find first cl > 0
idx = idx + 1; % Needs a +1 because we looked for cl(2:end)

if isempty(idx)
    if PLOT_POLAR_IF_ERROR
        figure('Name', 'Debug: Polar with no zero lift')
        plot(rad2deg(alpha), cl)
        title('Polar with no clear zero lift angle')
        xlabel('AOA [deg]')
        ylabel('C_l [-]')
        grid on
    end
    error('MATLAB:findzerolift:noPositiveCl',['Impossible to find a '...
        'positive Cl']);
end

if sign(cl(idx)) == sign(cl(idx-1))
    if PLOT_POLAR_IF_ERROR
        figure('Name', 'Debug: Polar with no zero lift')
        plot(rad2deg(alpha), cl)
        title('Polar with no clear zero lift angle')
        xlabel('AOA [deg]')
        ylabel('C_l [-]')
        grid on
    end
    error('MATLAB:findzerolift:noNegativeCl',['Impossible to find a '...
        'negative Cl']);
end

dummy(:,1) = alpha(idx-1:idx); % AOA of the two points around cl = 0
dummy(:,2) = cl(idx-1:idx); % Cl of the two points around cl = 0
alpha_zeroL = interp1(dummy(:,2),dummy(:,1), 0);

if nargin == 3
    cd_zeroL = interp1(alpha, cd, alpha_zeroL);
else
    cd_zeroL = NaN;
end

end