function [x, y] = nacaairfoil(digits, nPoints, spacing, finiteTrail)
% NACAAIRFOIL Generates the full coordinates of a NACA 4 or 5 digits airfoil.
%   By default, the airfoil is generated with 100 points distributed
%   following a half-cosine rule in order to be finer at the leading and
%   trailing edges. The default output has a zero thickness trailing edge.
%   Coefficients for a finite trailing edge are also implemented.
%   See below for details on how to override the defaults.
%
% Note:
%   This function will always output an odd number of points, even if the user 
%   asks for an even value of nPoints. This is because the upper and
%   lower surfaces will have the same number of points, and the leading
%   edge is counted twice (as the first and last element to close the shape)
%   while the trailing edge is counted once.
%
% <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox">Documentation (README)</a>
% <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox/-/issues">Report an issue</a>
% -----
%
% Synopsis:
%   [X, Y] = NACAAIRFOIL prompts the user the airfoil digits.
%
%   [X, Y] = NACAAIRFOIL(DIGITS) Calculates the coordinates of the NACA airfoil
%   specified by DIGITS, using a default value of 100 points spaced using the
%   halfcosine rule.
%
%   [X, Y] = NACAAIRFOIL(DIGITS, NPOINTS) Calculates the coordinates of the
%   the NACA airfoil specified by DIGITS, using NPOINTS points spaced using the
%   halfcosine rule.
%
%   [X, Y] = NACAAIRFOIL(DIGITS, NPOINTS, SPACING) Calculates the coordinates
%   of the NACA airfoil specified by DIGITS, using NPOINTS points spaced using
%   the SPACING rule.
%
%   [X, Y] = NACAAIRFOIL(DIGITS, NPOINTS, SPACING, FINITETRAIL) Calculates the
%   coordinates of the NACA airfoil specified by DIGITS, using NPOINTS points
%   spaced using the SPACING rule. Adapt the thickness of the trailing edge
%   according to FINITETRAIL.
%
% Inputs:
%   digits  : Character vector representing the NACA airfoil (ex: '0012', '24120', '24012')
%   nPoints : Number of output coordinates
%   spacing : Spacing of the airfoil coordinates ('linear', 'cosine', 'halfcosine')
%       - linear: linear spacing
%       - halfcosine: finer at tip and root (default)
%       - cosine: finer at tip only
%   finiteTrail : if the trailing edge is finite (true) or zero (false, default)
%
% Output:
%   [X, Y] : Airfoil coordinates, with 0 <= X <= 1.
%
% Example:
%   NACAAIRFOIL
%   NACAAIRFOIL('0012')
%   NACAAIRFOIL('24012', 120)
%   NACAAIRFOIL('24012', 120, 'halfcosine')
%   NACAAIRFOIL('24012', 120, 'linear', false)
%
% See also: NACACAMBER.

% -----
% Copyright 2022 Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
% ------------------------------------------------------------------------------


%% Import other functions from this package
import af_tools.nacacamber


%% Constants and defaults

DEFAULT_NPOINTS = 100;
DEFAULT_SPACING = 'halfcosine';
DEFAULT_FINITE_TE = false;

ALLOWED_SPACING = {'linear', 'cosine', 'halfcosine'};
MAX_PROMPTS = 3; % Number of re-promts before giving up


%% Check inputs and assign defaults

% Check for digits input, if not found, prompt user
if nargin == 0
    count = 1;
    digits = [];
    while isempty(digits) && count<= MAX_PROMPTS
        prompt = 'Input NACA 4 or 5 digits (ex: 0012, 24012): ';
        digits = input(prompt,'s');
        count = count+1;
    end
    if count >= MAX_PROMPTS
        fprintf(2,'No data input after 3 attempts. Operation aborded.\n');
        return
    end
end

validateattributes(digits, {'char','string'},{'scalartext','nonempty'}, mfilename(), ...
    'digits', 1)
if isstring(digits)
    digits = char(digits);
end

if length(digits) < 4 || length(digits) > 5
    error('MATLAB:nacaairfoil:wrongNumberOfDigits', ['nacaairfoil can only ' ...
        'be used for NACA 4 or 5 digits airfoils.']);
end

% Check for number of points
if nargin >= 2
    validateattributes(nPoints, {'numeric'},{'scalar','>=',2}, mfilename(), ...
        'nPoints', 2)
    % No need to warn user about too few points here as nacacamber will do it
else
    nPoints = DEFAULT_NPOINTS;
end

% Check for spacing
if nargin >= 3
    validateattributes(spacing, {'char','string'},{'scalartext'}, mfilename(), ...
        'spacing', 3)
    spacing = validatestring(spacing, ALLOWED_SPACING, mfilename(), 'spacing',3);
else
    spacing = DEFAULT_SPACING;
end

% Check for finiteTrail
if nargin >= 4
    validateattributes(finiteTrail, {'logical'},{'scalar'}, mfilename(), 'finiteTrail', 4)
else
    finiteTrail = DEFAULT_FINITE_TE;
end


%% Coordinates calculation

% Get camberline (we use ceil((nPoints+1)/2) to ensure the airfoil has the same
% number of points on the upper and lower surfaces.
[xc, yc, gradY] = nacacamber(digits, ceil((nPoints+1)/2), spacing);

% Thickness value (checks were done in nacacamber)
t = str2double(digits(end-1:end)) /100;

% Thickness coefficients
a0 = 0.2969;
a1 = -0.126;
a2 = -0.3516;
a3 = 0.2843;
if finiteTrail
    a4 = -0.1015;
else
    a4 = -0.1036;
end

% Add thickness
yt = t/0.2 * (a0*xc.^0.5 + a1*xc + a2*xc.^2 + a3*xc.^3 + a4*xc.^4);

% Get full coordinates
theta = atan(gradY);
xu = xc - yt.*sin(theta);
yu = yc + yt.*cos(theta);
xl = xc + yt.*sin(theta);
yl = yc - yt.*cos(theta);

x = [xu, fliplr(xl(1:end-1))];
y = [yu, fliplr(yl(1:end-1))];

end