function [xc, yc, gradY] = nacacamber(digits, nPoints, spacing)
% NACACAMBER Generates the coordinates of the camberline for a NACA 4 or 5 digits airfoil.
%   By default, the camberline is generated with 100 points distributed
%   following a half-cosine rule in order to be finer at the leading and
%   trailing edges.
%   See below for details on how to override the defaults.
%
% <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox">Documentation (README)</a>
% <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox/-/issues">Report an issue</a>
% -----
%
% Synopsis:
%   [XC, YC] = NACACAMBER prompts the user the airfoil digits.
%
%   [XC, YC] = NACACAMBER(DIGITS) Calculates the coordinates of the camberline
%   for the NACA airfoil specified by DIGITS, using a default value of 100
%   points spaced using the halfcosine rule.
%
%   [XC, YC] = NACACAMBER(DIGITS, NPOINTS) Calculates the coordinates of the
%   camberline for the NACA airfoil specified by DIGITS, using NPOINTS points
%   spaced using the halfcosine rule.
%
%   [XC, YC] = NACACAMBER(DIGITS, NPOINTS, SPACING) Calculates the coordinates
%   of the camberline for the NACA airfoil specified by DIGITS, using NPOINTS
%   points spaced using the SPACING rule.
%
% Inputs:
%   digits  : Character vector representing the NACA airfoil (ex: '0012', '24120', '24012')
%   nPoints : Number of output coordinates
%   spacing : Spacing of the airfoil coordinates ('linear', 'cosine', 'halfcosine')
%       - linear: linear spacing
%       - halfcosine: finer at tip and root (default)
%       - cosine: finer at tip only
%
% Output:
%   xc : Camberline X coordinates (0 <= X <= 1)
%   yc : Camberline Y coordinates
%   gradY : Gradient of Y (required to get the whole airfoil coordinates)
%
% Example:
%   NACACAMBER
%   NACACAMBER('0012')
%   NACACAMBER('24012', 120)
%   NACACAMBER('24012', 120, 'halfcosine')
%
% See also: NACAAIRFOIL.

% -----
% Copyright 2022 Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
% ------------------------------------------------------------------------------


%% Constants and defaults

DEFAULT_NPOINTS = 100;
DEFAULT_SPACING = 'halfcosine';

ALLOWED_SPACING = {'linear', 'cosine', 'halfcosine'};
MAX_PROMPTS = 3; % Number of re-promts before giving up


%% Check inputs and assign defaults


% Check for digits input, if not found, prompt user
if nargin == 0
    count = 1;
    digits = [];
    while isempty(digits) && count<= MAX_PROMPTS
        prompt = 'Input NACA 4 or 5 digits (ex: 0012, 24012): ';
        digits = input(prompt,'s');
        count = count+1;
    end
    if count >= MAX_PROMPTS
        fprintf(2,'No data input after 3 attempts. Operation aborded.\n');
        return
    end
end

validateattributes(digits, {'char','string'},{'scalartext'}, mfilename(), ...
    'digits', 1)
if isstring(digits)
    digits = char(digits);
end

if length(digits) < 4 || length(digits) > 5
    error('MATLAB:nacacamber:wrongNumberOfDigits', ['nacacamber can '...
        'only be used for NACA 4 or 5 digits airfoils.']);
end

% Check for number of points
if nargin >= 2
    validateattributes(nPoints, {'numeric'},{'scalar','>=', 2}, mfilename(), ...
        'nPoints', 2)
    if nPoints <=10
        warning('MATLAB:nacacamber:lowNPoints',['The number of points is '...
            'very low (%d), the camberline may be poorly defined.'], nPoints);
    end
else
    nPoints = DEFAULT_NPOINTS;
end

% Check for spacing
if nargin >= 3
    validateattributes(spacing, {'char','string'},{'scalartext'}, mfilename(), ...
        'spacing', 3)
    spacing = validatestring(spacing, ALLOWED_SPACING, mfilename(), 'spacing',3);
else
    spacing = DEFAULT_SPACING;
end


%% Coordinates calculation


if length(digits) == 4
    [xc, yc, gradY] = camber4digits(digits, nPoints, spacing);
elseif length(digits) == 5
    [xc, yc, gradY] = camber5digits(digits, nPoints, spacing);
end

end


%% Helpers

function [xc, yc, gradY] = camber4digits(digits, nPoints, spacing)
% CAMBER4DIGITS Give the camberline coordinates for a 4 digits airfoil.
%
% NACA Definition:  NACA MPXX
%   M  = Maximum camber divided by 100
%   P  = Position of the max camber divided by 10
%   XX = Maximum thickness divided by 100

% Airfoil parameters
m = str2double(digits(1)) /100;
p = str2double(digits(2)) /10;
t = str2double(digits(end-1:end)) /100;

if isnan(m) || isnan(p) || isnan(t)
    error('MATLAB:nacacamber:invalidDigits', ['The input (''%s'') does not '...
        'correspond to a proper NACA airfoil'], digits);
end


% Camberline
xc = getxc(spacing, nPoints);
yc = zeros(size(xc));
gradY = zeros(size(xc));

if m ~= 0
    idx = find(xc <= p); % Front part
    yc(idx) = m/p^2 * (2*p*xc(idx) - xc(idx).^2);
    gradY(idx) = 2*m/p^2 * (p-xc(idx));
    
    idx = find(xc > p); % Aft part
    yc(idx) = m/(1-p)^2 * (1 - 2*p + 2*p*xc(idx)-xc(idx).^2);
    gradY(idx) = 2*m/(1-p)^2 * (p-xc(idx));
end

end

function [xc, yc, gradY] = camber5digits(digits, nPoints, spacing)
% CAMBER5DIGITS Give the camberline coordinates for a 5 digits airfoil.
%
% NACA Definition:  NACA LPQXX
%   L = Design lift coefficient multiplied by 3/20
%   P = Position of the max camber divided by 20 (ex 3 is 0.15*chord)
%   Q = Camber line (0=standard, 1=reflex)
%   XX = Maximum thickness divided by 100


% Airfoil parameters
cld = str2double(digits(1)) *3/20;
p = str2double(digits(2)) /20;
q = str2double(digits(3));
t = str2double(digits(end-1:end)) /100;

% Check if proper specification
if isnan(cld) || isnan(p) || isnan(q) || isnan(t)
    error('MATLAB:nacacamber:invalidDigits', ['The input (''%s'') does not '...
        'correspond to a proper NACA airfoil'], digits);
end
if q ~= 0 && q ~= 1
    error('MATLAB:nacacamber:invalid5Digits', ['The third digit of a NACA 5 '...
        'series must either be a 0 (standard camber) or a 1 (reflex camber). '...
        'Found %d'], q);
end

% Camberline
xc = getxc(spacing, nPoints);
yc = zeros(size(xc));
gradY = zeros(size(xc));

if q == 0 % Standard camber formula
    if p ~= 0
        [r, k1, ~] = getcoeffs5digits(q, p);
        idx = find(xc <= r); % Front part
        yc(idx) = k1/6 * (xc(idx).^3 - 3*r*xc(idx).^2 + r^2*(3-r)*xc(idx));
        gradY(idx) = k1/6 * (3*xc(idx).^2 - 6*r*xc(idx) + r^2*(3-r));
        
        idx=find(xc > r); % Aft part
        yc(idx) = k1*r^3/6 * (1-xc(idx));
        gradY(idx) = -k1*r^3/6;
    end
elseif q == 1 % Reflex camber formula
    if p ~= 0
        [r, k1, k2k1] = getcoeffs5digits(q, p);
        idx = find(xc <= r); % Front part
        yc(idx) = k1/6 * ((xc(idx)-r).^3 - k2k1*(1-r)^3*xc(idx) - r^3*xc(idx) + r^3);
        
        idx=find(xc > r); % Aft part
        yc(idx) = k1/6 * (k2k1*(xc(idx)-r).^3 - k2k1*(1-r)^3*xc(idx) - r^3*xc(idx) + r^3);
    end
end

% Scale yc according to cld
yc = (cld/0.3) * yc;

end


function xc = getxc(spacing, nPoints)
% GETXC Returns xc vector based on chosen spacing and number of points

switch spacing
    case 'linear'
        xc = linspace(0, 1, (nPoints));
    case 'halfcosine'
        beta = linspace(0, pi, (nPoints));
        xc = (1-cos(beta))/2;
    case 'cosine'
        beta = linspace(0, pi/2, (nPoints));
        xc = 1-cos(beta);
end

end

function [r,k1,k2k1] = getcoeffs5digits(q, p)
% GETCOEFFS5DIGITS Outputs the value of the various coefficients used in the NACA 5digits formula.
% Source: http://www.airfoiltools.com/airfoil/naca5digit

INTERP_METHOD = 'spline';

if q == 0 % Stanard camber
    coeffs = [0.05, 0.0580, 361.400;
        0.10, 0.1260, 51.640;
        0.15, 0.2025, 15.957;
        0.20, 0.2900, 6.643;
        0.25, 0.3910, 3.230;];
    k2k1 = NaN;
    
elseif q == 1 % Reflex camber
    coeffs = [0.10, 0.1300, 51.990, 0.000764;
        0.15, 0.2170, 15.793, 0.00677;
        0.20, 0.3180, 6.650, 0.0303;
        0.25, 0.4410, 3.191, 0.1355;];
    k2k1 = interp1(coeffs(:,1), coeffs(:,end), p, INTERP_METHOD, 'extrap');
end

r = interp1(coeffs(:,1), coeffs(:,2), p, INTERP_METHOD, 'extrap');
k1 = interp1(coeffs(:,1), coeffs(:,3), p, INTERP_METHOD, 'extrap');

end