function plotpolars(alpha, cl, cd, cm)
% PLOTPOLARS Plot various polars
%   This function can either take a full Polar structure (generated with
%   xf2mat) or values for angles of attack and their associated cl and cd.
%
%   If the input is specified as aoa, cl, cd and cm, the four arrays must have the 
%   same number of rows (one for each angle of attack). The columns indicate 
%   values for different polars (different Mach, different Reynolds,...). 
%   For example, if the cl values are given for 2 different Reynols and there
%   are 100 values for the aoa, cl, cd and cm must be [100 x 2].
%
% Plots:
%   - Cl - aoa
%   - Cd - aoa
%   - Cm - aoa
%   - Cl/Cd - aoa
%   - Cl - Cd
%
% <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox">Documentation (README)</a>
% <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox/-/issues">Report an issue</a>
% -----
%
% Synopsis:
%   PLOTPOLARS prompts the user to input a Polar structure then plots the polars.
%
%   PLOTPOLARS(POLAR) plots the polar found in the POLAR structure.
%
%   PLOTPOLARS(ALPHA, CL) plots the CL - ALPHA polars
%
%   PLOTPOLARS(ALPHA, CL, CD) plots the CL - ALPHA, CD - ALPHA, CL/CD - ALPHA
%   and CL-CD polars.
%
%   PLOTPOLARS(ALPHA, CL, CD, CM) plots the CL - ALPHA, CD - ALPHA, CM - ALPHA,
%   CL/CD - ALPHA and CL-CD.
%
% Inputs:
%   Polar : Polar structure
%   alpha : Vector with angles of attack
%   cl    : Array with lift coefficients [aoas x nbPolar]
%   cd    : Array with drag coefficients [aoas x nbPolar]
%   cm    : Array with moment coefficients [aoas x nbPolar]
%
% Output:
%   Various plots
%
% Example:
%   PLOTPOLARS
%   PLOTPOLARS(Polar)
%   PLOTPOLARS(alpha, cl)
%   PLOTPOLARS(alpha, cl, cd)
%   PLOTPOLARS(alpha, cl, cd, cm)
%
% See also: XF2MAT.

% -----
% Copyright 2022 Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
% ------------------------------------------------------------------------------


%% Import other functions from this package
import af_tools.utils.*


%% Check inputs

% If no arguments, prompt user to input a Polar file
if nargin == 0
    [filename, path] = uigetfile('*.mat', 'Select a Polar file');
    tmp = load([path,filename]);
    Polar = tmp.Polar;
    clear tmp;
end

if nargin == 1
    % If one single argument reassign variable properly
    Polar = alpha;
    clear alpha;
end

% Validate Polar structure
if nargin <= 1
    validateattributes(Polar, {'struct'},{'nonempty'}, mfilename(), 'Polar', 1)
    [alpha, cl, cd, cm] = extractpolar(Polar);
    clear Polar;
end

% Validate alpha
alpha = vecttocol(alpha);
validateattributes(alpha, {'numeric'},{'nonempty'}, mfilename(), ...
    'alpha')

% Validate cl
cl = vecttocol(cl);
validateattributes(cl, {'numeric'},{'nrows', size(alpha,1)}, mfilename(),...
    'cl')

% Validate cd
if nargin ~= 2
    cd = vecttocol(cd);
    validateattributes(cd, {'numeric'},{'nrows', size(alpha,1)}, mfilename(),...
        'cd')
end

% Validate cm
if nargin ~= 2 && nargin ~= 3
    cm = vecttocol(cm);
    validateattributes(cm, {'numeric'},{'nrows', size(alpha,1)}, mfilename(),...
        'cm')
end

% Standardize alpha size so we have one column per polar as well
if size(alpha,2) == 1
    alpha = repmat(alpha, [1, size(cl,2)]);
end


%% Plots

set(0, 'defaultTextInterpreter', 'latex'); % Default interpreter for all plots

plotpolar(rad2deg(alpha), cl, '$C_l - \alpha$', '$\alpha$ [deg]', '$C_l$ [-]');
if nargin > 2 || nargin <= 1
    plotpolar(rad2deg(alpha), cd, '$C_d - \alpha$', '$\alpha$ [deg]', '$C_d$ [-]');
    plotpolar(rad2deg(alpha), cl./cd, '$C_l/C_d - \alpha$', '$\alpha$ [deg]', '$C_l/C_d$ [-]');
    plotpolar(cl, cd, '$C_d - C_l$', '$C_l$ [-]', '$C_d$ [-]');
end
if nargin > 3 || nargin <= 1
    plotpolar(rad2deg(alpha), cm, '$C_m - \alpha$', '$\alpha$ [deg]', '$C_m$ [-]');
end


end


%% Helpers

function plotpolar(dataX, dataY, fTitle, fXlabel, fYlabel)
% PLOTPOLAR Standardize polar plots

if ~exist('fTitle','var')
    fTitle = '';
end
figure('PaperUnits', 'inches', 'PaperPosition', [0 0 1280 1024]/250, 'Name',fTitle);
plot(dataX,dataY)

if exist('fXlabel','var')
    xlabel(fXlabel);
end
if exist('fYlabel','var')
    ylabel(fYlabel);
end

setgca()
end

function setgca()
% SETGCA Sets axes options for all plots

grid on
set(gca, 'FontName','Helvetica');
set(gca, 'Fontsize', 14);
set(gca,...
    'Box', 'off',...
    'TickDir'     , 'out'     , ...
    'TickLength'  , [.02 .02] , ...
    'XMinorTick'  , 'off'      , ...
    'YMinorTick'  , 'off'      , ...
    'YGrid'       , 'on'      , ...
    'XGrid'       , 'on'      , ...
    'XColor'      , 'k', ...
    'YColor'      , 'k', ...
    'GridLineStyle', ':'         , ...
    'GridColor', 'k', ...
    'GridAlpha', 0.25, ...
    'LineWidth'   , 1         );
set(gcf, 'PaperPositionMode', 'auto');

end