# Contributing

When contributing to this repository, please first discuss the change you wish
to make via issue, email, or any other before making a change.

## Code standards

All changes submitted must conform to the following code standards.

### Main functions

The so called _Main functions_ are the ones present in the root of `+af_tools/`.
These are the core functions of the package.

- Every main function should start with a lengthy comment that details its use.
  The first line of the comment must be a one-line description of the function
  (this line is searched when the user type `lookfor <keyword>` in Matlab). The
  next block of continuous comment is printed when the user types `help
  <function>`. As this is the only documentation provided for the different
  functions, this comment should describe the whole function, as well as all
  inputs and outputs. Some examples of use should also be provided. If some
  special design choices were made for the implementation, they should be
  documented as well.
- Every main function should validate the user inputs (using
  `validateattributes`, `validatestring`, `try/catch` blocks, etc) to ensure
  proper use.
- All main functions should return a comprehensive error message in case of
  failure. If applicable, warnings should be returned if the inputs/outputs are
  expected to be of poor quality due to poor, but valid, input.
- If possible, the main functions should be coded so they can be run without any
  input argument. If some argument is absolutely mandatory, the users should be
  prompted through the command window or a dialog box. All other arguments
  should be set at proper defaults values.
- Each main function should be standalone tool that makes a complete operation.
  Sub-routines for intermediary tasks are allowed, but must be placed in
  `+af_tools/+utils/` (see below for details regarding sub-routines).
- The functions must be inter-operable with little effort from the user
  (function B reuses the output of function A without much modification).
- The functions can be called directly from one another. In that case, the
  import statements must be the first lines of code in the function.
- Each function should come with a set of tests (in `tests/`) to ensure all
  errors are raised and standard outputs can always be reproduced.
- The functions should not contain commented code.
- Code should be properly formatted by Matlab (select all, then `ctrl`+ `i`)

### Sub-routines

The so called _Sub-routines_ are small chunks of code that do not bear great
interest in themselves, but are shared by different _Main functions_. To
separate them clearly, they must be placed in `+af_tools/+utils/`.

- The sub-routines do not require a long help comment. A one line description is
  enough.
- The sub-routines are not made to be used directly by the end user. As such,
  there is no need for complex input validation.
- Sub-routines must also come with a set of tests (in `test/`). The error
  verification tests can be skipped, so the test function only contains tests
  for the expected outputs.
- The functions should not contain commented code.
- Code should be properly formatted by Matlab (select all, then `ctrl`+ `i`)

## Merge Request Process

1. Ensure the **code standards** are properly respected (function description,
   help comment, unit tests, inter-operability, etc).
2. Ensure all tests pass in Matlab (`runtests('tests')`).
3. Update the CHANGELOG.md and CODEOWNERS to document the changes.
4. Update the README.md to reflect the changes if applicable.
5. Submit your Merge Request.
