# Airfoil toolbox

_A list of functions and script used for analysis of airfoils and related data
with Matlab._

## Disclaimer

This toolbox was mainly developed under Matlab R2018a. The various functions
were not tested on older versions. A code compatibility analysis showed no
errors, so _in theory_ it should work properly with any other versions of Matlab
as well. If for some reason it does not work on your version, please [open an
issue][gitlab_issues].

The functions in this repository are intended to be used as a _Matlab package_
(see [Matlab documentation][matlabDoc_packages] for more details).

## Documentation

Each function contains a complete description in its preamble. Type `help
<function>` in Matlab command window to print it.

## Install and use

### Installation

1. Either clone this repository or download a `.zip` version of the source code.
2. Place the `+af_tools/` folder somewhere on your Matlab Path such as:

- in you current project
- in a folder that is already in Matlab's path (e.g. `/Documents/Matlab/`).
- somewhere else, but then [explicitly add it to Matlab's Path][matlabDoc_path].

#### Remark

Unfortunately, Matlab packages have a few limitations. Hence, you should **avoid
changing the package name** if you do not know what you are doing.

- The `+` in the folder's name is required for Matlab to treat it as a package.
- Some functions of this package are interconnected (_e.g._, _nacairfoil_ calls
  _nacacamber_). In that situation, Matlab forces the main function to re-import
  explicitly all sub-functions before using them, even if they are part of the
  same package ([people have complained][matlabPackgeIssue] about that design
  choice for quite some time, to no avail).
  If you rename the package, these links will be broken and Matlab will not find
  the connected functions.
- All functions that import other functions will have an import line at the
  beginning. If you rename the package, you must therefore update these imports.

### Use

Once the `+af_tools/` folder is placed somewhere where Matlab will find
it, you can start using its features.

- If you are only interested in a single function of the package, use

```matlab
import af_tools.<functionName>
functionName(args)
```

- If you want to import all functions, use

```matlab
import af_tools.*
xf2mat(args)
nacacamber(args)
...
```

- If you want to use functions on-the-fly without importing them, call them
  directly

```matlab
af_tools.xf2mat(args)
```

## Functions

- All functions can be used as standalone functions and give meaningful output
  on their own (they are not sub-routines used to perform intermediary tasks).
- Most functions can be run without input arguments. In that case, the functions
  will prompt the user for the few mandatory arguments while the others will be
  set to their default values.

### xf2mat

Aggregates airfoil polars obtained using [XFOIL v6.99][xfoil] of [XFLR5
v6.55][xflr5] into a single Matlab structure.

This function can either aggregate multiple polar files into a single structure
(_e.g._ same airfoil at different Reynolds) or simply convert a single polar
data into a Matlab-ready structure.

```matlab
Polar = xf2mat
Polar = xf2mat(autosave, trimAoas, inputDir, inputFiles)
```

| Input        | Example                                        | Default |
|------------- | ---------------------------------------------- | --------|
| `autosave`   | `false`, `true`                                | false   |
| `trimAoas`   | `false`, `true`                                | -       |
| `inputDir`   | `'.'`, `'xf_results'`                          | -       |
| `inputFiles` | `'*'`, `'*0012*'`, `'{'*0012_1*', '*0012_2*'}` | false   |

The output is a structure with the following fields, where M is the number of
angles of attack and N is the number of input files.

| Field        | Data                                             | Size    |
|------------- | ------------------------------------------------ | --------|
| `origin`     | Program from which the polars came (XFOIL/XFLR5) | [1 x N] |
| `airfoil`    | Airfoil name (parsed from XFOIL/XFLR5 results)   | [1 x N] |
| `alpha`      | Angles of attacks                                | [M x N] |
| `reynolds`   | Reynolds numbers                                 | [1 x N] |
| `mach`       | Mach number                                      | [1 x N] |
| `nCrit`      | BL transition factor                             | [1 x N] |
| `cl`         | Lift coefficient                                 | [M x N] |
| `cd`         | Drag coefficient                                 | [M x N] |
| `cm`         | Moment coefficient                               | [M x N] |

### extendpolar

This function extends known $`C_l`$ and $`C_d`$ polars to the full range of
angle of attacks [-180; 180]. This is especially useful when studying rotors and
propellers in off-design configuration (with the Blade Element Momentum Theory
for instance).

```matlab
[alphaExt, clExt, cdExt] = extendpolar
[alphaExt, clExt, cdExt] = extendpolar(method, limit, Polar)
[alphaExt, clExt, cdExt] = extendpolar(method, limit, alpha, cl, cd)
```

| Input         | Example                            | Default |
|-------------- | ---------------------------------- | ------- |
| `method`      | `'viterna'`                        | -       |
| `limit`       | `'stall','limit'`                  | -       |
| `Polar`       | `MyPolar`                          | -       |
| `alpha`       | `1:10`                             | -       |
| `cl`          | `1:10`, `rand(10,2)`               | -       |
| `cd`          | `1:10`, `rand(10,2)`               | -       |

### nacacamber

Determines the camber line of a NACA 4 or 5 digits airfoil.

```matlab
[xc, yc, gradY] = nacacamber
[xc, yc, gradY] = nacacamber(digits, nPoints, spacing)
```

| Input     | Example                            | Default      |
|---------- | ---------------------------------- | ------------ |
| `digits`  | `'0012'`, `'24012'`                | -            |
| `nPoints` | `100`, `130`                       | `100`        |
| `spacing` | `halfcosine`, `cosine`, `linear`   | `halfcosine` |

### nacaairfoil

Generates the full coordinates of a NACA 4 or 5 digits airfoil.

```matlab
[x, y] = nacaairfoil
[x, y] = nacaairfoil(digits, nPoints, spacing, finiteTrail)
```

| Input         | Example                            | Default      |
|-------------- | ---------------------------------- | ------------ |
| `digits`      | `'0012'`, `'24012'`                | -            |
| `nPoints`     | `100`, `130`                       | `100`        |
| `spacing`     | `halfcosine`, `cosine`, `linear`   | `halfcosine` |
| `finiteTrail` | `true`, `false`                    | `false`      |

### plotpolars

Plots the different polars:

- $`C_l--\alpha`$
- $`C_d--\alpha`$
- $`C_m--\alpha`$
- $`C_l/C_d--\alpha`$
- $`C_l--C_d`$

```matlab
plotpolars
plotpolars(Polar)
plotpolars(alpha, cl, cd, cm)
```

| Input         | Example                            | Default |
|-------------- | ---------------------------------- | ------- |
| `Polar`       | `MyPolar`                          | -       |
| `alpha`       | `1:10`                             | -       |
| `cl`          | `1:10`, `rand(10,2)`               | -       |
| `cd`          | `1:10`, `rand(10,2)`               | -       |
| `cm`          | `1:10`, `rand(10,2)`               | -       |

### findstall

Finds the stall angle of attack and the associated $`C_l`$ and $`C_d`$.

```matlab
[alpha_s, cl_s, cd_s] = findstall
[alpha_s, cl_s, cd_s] = findstall(Polar)
[alpha_s, cl_s, cd_s] = findstall(alpha, cl, cd)
```

| Input         | Example                            | Default |
|-------------- | ---------------------------------- | ------- |
| `Polar`       | `MyPolar`                          | -       |
| `alpha`       | `1:10`                             | -       |
| `cl`          | `1:10`, `rand(10,2)`               | -       |
| `cd`          | `1:10`, `rand(10,2)`               | -       |

### findzerolift

Finds the zero-lift angle of attack and the associated $`C_d`$.

```matlab
[alpha_zeroL, cd_zeroL] = findzerolift
[alpha_zeroL, cd_zeroL] = findzerolift(Polar)
[alpha_zeroL, cd_zeroL] = findzerolift(alpha, cl, cd)
```

| Input         | Example                            | Default |
|-------------- | ---------------------------------- | ------- |
| `Polar`       | `MyPolar`                          | -       |
| `alpha`       | `1:10`                             | -       |
| `cl`          | `1:10`, `rand(10,2)`               | -       |
| `cd`          | `1:10`, `rand(10,2)`               | -       |

### findcllinearrange

Finds the range of angles of attack for which the $`C_l`$ can be considered
linear.

```matlab
[alphaRange, clRange, clSlope] = findcllinearrange
[alphaRange, clRange, clSlope] = findcllinearrange(Polar)
[alphaRange, clRange, clSlope] = findcllinearrange(alpha,cl)
```

| Input         | Example                            | Default |
|-------------- | ---------------------------------- | ------- |
| `Polar`       | `MyPolar`                          | -       |
| `alpha`       | `1:10`                             | -       |
| `cl`          | `1:10`, `rand(10,2)`               | -       |

## Licence

Copyright (c) 2022 [Thomas Lambert][tlambert], University of Liège.

All functions and scripts of the **af_tools** code are licensed under the
[Apache 2.0 Licence](LICENCE).

[xfoil]: http://web.mit.edu/drela/Public/web/xfoil/
[xflr5]: https://www.xflr5.tech/xflr5.htm
[tlambert]: <https://tlambert.xyz>
[gitlab_issues]: https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox/-/issues
[matlabDoc_packages]: <https://www.mathworks.com/help/matlab/matlab_oop/scoping-classes-with-packages.html>
[matlabDoc_path]: <https://www.mathworks.com/help/matlab/matlab_env/what-is-the-matlab-search-path.html>
[matlabPackgeIssue]: https://www.mathworks.com/matlabcentral/answers/280234-misleading-wording-in-the-documentation-on-matlab-package-folders
