% TEST_EXTENDPOLAR Unitary tests for the extendpolar function
% Note:
%   The function needs to be imported at each test, because apparently Matlab
%   does not pass it from setupOnce() or setup().
%   For some reasons, the whole package needs to be imported, not just the
%   function under testing.

% -----
% Copyright 2022 Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
% ------------------------------------------------------------------------------


%% Main test function

function tests = test_extendpolar

tests = functiontests(localfunctions);

end


%% Setup and teardown

function setupOnce(testCase)  % do not change function name

addpath('../.'); % Add repository to Matlab Path
addpath('./test_utils'); % Add utils to Matlab Path

% Set random number generator settings.
testCase.TestData.currentRNG = rng;

end

function teardownOnce(testCase)  % do not change function name

rmpath('../.'); % Remove repository from Matlab Path
rmpath('./test_utils'); % Remove utils from Matlab Path

% Restore the random number generator settings.
s = testCase.TestData.currentRNG;
rng(s)

end

function teardown(~)  % do not change function name

close all; % Close all figures that would have been openend

end


%% Arguments and errors tests

function test_nargin(testCase)
% Ensure that the function throws an error when called with too many arguments

import af_tools.*

dummy = (1:10)';

verifyError(testCase, @() extendpolar(dummy, dummy, dummy, dummy, dummy, dummy), 'MATLAB:TooManyInputs')
verifyError(testCase, @() extendpolar(dummy, dummy, dummy, dummy), 'MATLAB:extendpolar:wrongNumberOfInputs')

end


function test_invalidInputs(testCase)
% Ensure that the function throws an exception if invalid arguments (type or size).

import af_tools.*

METHOD = 'viterna';
LIMIT = 'last';

empty = [];
scal = 0;
logi = true;
char = 'test';

vectPos = 1:10;
vect10n = 0:9;
vect11n = 0:10;

% If 1 input only, it should be the correct method
verifyError(testCase, @() extendpolar(empty), 'MATLAB:extendpolar:invalidType')
verifyError(testCase, @() extendpolar(scal), 'MATLAB:extendpolar:invalidType')
verifyError(testCase, @() extendpolar(logi), 'MATLAB:extendpolar:invalidType')
verifyError(testCase, @() extendpolar(char), 'MATLAB:extendpolar:unrecognizedStringChoice')

% If 2 inputs only, 2nd one should be the correct limit
verifyError(testCase, @() extendpolar(METHOD, empty), 'MATLAB:extendpolar:invalidType')
verifyError(testCase, @() extendpolar(METHOD, scal), 'MATLAB:extendpolar:invalidType')
verifyError(testCase, @() extendpolar(METHOD, logi), 'MATLAB:extendpolar:invalidType')
verifyError(testCase, @() extendpolar(METHOD, char), 'MATLAB:extendpolar:unrecognizedStringChoice')

% If 3 inputs, third one should be a Polar strcuture
verifyError(testCase, @() extendpolar(METHOD, LIMIT, empty), 'MATLAB:extendpolar:invalidType')
verifyError(testCase, @() extendpolar(METHOD, LIMIT, scal), 'MATLAB:extendpolar:invalidType')
verifyError(testCase, @() extendpolar(METHOD, LIMIT, logi), 'MATLAB:extendpolar:invalidType')
verifyError(testCase, @() extendpolar(METHOD, LIMIT, char), 'MATLAB:extendpolar:invalidType')

% If 5 inputs, inputs 3, 4 and 5 should be vectors or arrays of numerics
verifyError(testCase, @() extendpolar(METHOD, LIMIT, empty, vect10n, vect10n), 'MATLAB:extendpolar:expectedNonempty')
verifyError(testCase, @() extendpolar(METHOD, LIMIT, logi, vect10n, vect10n), 'MATLAB:extendpolar:invalidType')
verifyError(testCase, @() extendpolar(METHOD, LIMIT, char, vect10n, vect10n), 'MATLAB:extendpolar:invalidType')
verifyError(testCase, @() extendpolar(METHOD, LIMIT, vect10n, logi, vect10n), 'MATLAB:extendpolar:invalidType')
verifyError(testCase, @() extendpolar(METHOD, LIMIT, vect10n, char, vect10n), 'MATLAB:extendpolar:invalidType')
verifyError(testCase, @() extendpolar(METHOD, LIMIT, vect10n, vect10n, logi), 'MATLAB:extendpolar:invalidType')
verifyError(testCase, @() extendpolar(METHOD, LIMIT, vect10n, vect10n, char), 'MATLAB:extendpolar:invalidType')

% Polars should include the value for AOA = 0deg
verifyError(testCase, @() extendpolar(METHOD, LIMIT, vectPos, vectPos, vectPos), 'MATLAB:extendpolar:noZero')

% Polars should include the value for AOA = 0deg
verifyError(testCase, @() extendpolar(METHOD, LIMIT, vect10n-1, vect10n-1, vect10n-1), 'MATLAB:extendpolar:noStall')

% Inputs 4 and 5 must have same length as 3
verifyError(testCase, @() extendpolar(METHOD, LIMIT, vect10n, vect11n, vect10n),'MATLAB:extendpolar:incorrectNumrows')
verifyError(testCase, @() extendpolar(METHOD, LIMIT, vect10n, vect10n, vect11n), 'MATLAB:extendpolar:incorrectNumrows')

end


function test_invalidPolarStructure(testCase)
% Ensure that the function throws an exception if Polar structure does not
% contain expected fields

import af_tools.*

METHOD = 'viterna';
LIMIT = 'last';

Dummy = dummylinearpolar();
DummyAlpha = rmfield(Dummy, 'alpha');
DummyCl = rmfield(Dummy, 'cl');
DummyCd = rmfield(Dummy, 'cd');

verifyError(testCase, @() extendpolar(METHOD, LIMIT, DummyAlpha), 'MATLAB:nonExistentField')
verifyError(testCase, @() extendpolar(METHOD, LIMIT, DummyCl), 'MATLAB:nonExistentField')
verifyError(testCase, @() extendpolar(METHOD, LIMIT, DummyCd), 'MATLAB:nonExistentField')

end


function test_invalidPolarAlpha(testCase)
% Ensure that the function throws an exception if Polar alpha is incorrect

import af_tools.*

METHOD = 'viterna';
LIMIT = 'last';

Dummy = dummylinearpolar();
DummyE = Dummy; DummyE.alpha = [];
DummyL = Dummy; DummyL.alpha = true;
DummyC = Dummy; DummyC.alpha = 'test';

verifyError(testCase, @() extendpolar(METHOD, LIMIT, DummyE), 'MATLAB:extendpolar:expectedNonempty')
verifyError(testCase, @() extendpolar(METHOD, LIMIT, DummyL), 'MATLAB:extendpolar:invalidType')
verifyError(testCase, @() extendpolar(METHOD, LIMIT, DummyC), 'MATLAB:extendpolar:invalidType')

end


function test_invalidPolarCl(testCase)
% Ensure that the function throws an exception if Polar cl is incorrect

import af_tools.*

METHOD = 'viterna';
LIMIT = 'last';

Dummy = dummylinearpolar();
DummyL = Dummy; DummyL.cl = true;
DummyC = Dummy; DummyC.cl = 'test';

Dummy11 = Dummy;
Dummy11.cl = 0:11;

verifyError(testCase, @() extendpolar(METHOD, LIMIT, DummyL), 'MATLAB:extendpolar:invalidType')
verifyError(testCase, @() extendpolar(METHOD, LIMIT, DummyC), 'MATLAB:extendpolar:invalidType')

verifyError(testCase, @() extendpolar(METHOD, LIMIT, Dummy11), 'MATLAB:extendpolar:incorrectNumrows')

end


function test_invalidPolarCd(testCase)
% Ensure that the function throws an exception if Polar cd is incorrect

import af_tools.*

METHOD = 'viterna';
LIMIT = 'last';

Dummy = dummylinearpolar();
DummyL = Dummy; DummyL.cd = true;
DummyC = Dummy; DummyC.cd = 'test';

Dummy11 = Dummy;
Dummy11.cd = 0:11;

verifyError(testCase, @() extendpolar(METHOD, LIMIT, DummyL), 'MATLAB:extendpolar:invalidType')
verifyError(testCase, @() extendpolar(METHOD, LIMIT, DummyC), 'MATLAB:extendpolar:invalidType')

verifyError(testCase, @() extendpolar(METHOD, LIMIT, Dummy11), 'MATLAB:extendpolar:incorrectNumrows')

end


function test_noStall(testCase)
% Ensure that the function throws an exception if no stall was found

import af_tools.*

METHOD = 'viterna';
LIMIT = 'last';

Polar = dummylinearpolar();

verifyError(testCase, @() extendpolar(METHOD, LIMIT, Polar), 'MATLAB:extendpolar:noStall')

end