% TEST_FINDCLLINEARRANGE Unitary tests for the findcllinearrange function
% Note:
%   The function needs to be imported at each test, because apparently Matlab
%   does not pass it from setupOnce() or setup().
%   For some reasons, the whole package needs to be imported, not just the
%   function under testing.

% -----
% Copyright 2022 Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
% ------------------------------------------------------------------------------


%% Main test function

function tests = test_findcllinearrange

tests = functiontests(localfunctions);

end


%% Setup and teardown

function setupOnce(testCase)  % do not change function name

addpath('../.'); % Add repository to Matlab Path
addpath('./test_utils'); % Add utils to Matlab Path

% Set random number generator settings.
testCase.TestData.currentRNG = rng;

end

function teardownOnce(testCase)  % do not change function name

rmpath('../.'); % Remove repository from Matlab Path
rmpath('./test_utils'); % Remove utils from Matlab Path

% Restore the random number generator settings.
s = testCase.TestData.currentRNG;
rng(s)

end

function teardown(~)  % do not change function name

close all; % Close all figures that would have been openend

end


%% Arguments and errors tests

function test_nargin(testCase)
% Ensure that the function throws an error when called with too many arguments

import af_tools.*
dummy = (1:10)';

verifyError(testCase, @() findcllinearrange(dummy, dummy, 1), 'MATLAB:TooManyInputs')

end


function test_invalidInputs(testCase)
% Ensure that the function throws an exception if invalid arguments (type or size).

import af_tools.*

empty = [];
scal = 0;
logi = true;
char = 'test';
vect2n = 0:1;
vect10n = 1:10;
vect11n = 1:11;

% If 1 input only, it should be a struct
verifyError(testCase, @() findcllinearrange(empty), 'MATLAB:findcllinearrange:invalidType')
verifyError(testCase, @() findcllinearrange(scal), 'MATLAB:findcllinearrange:invalidType')
verifyError(testCase, @() findcllinearrange(logi), 'MATLAB:findcllinearrange:invalidType')
verifyError(testCase, @() findcllinearrange(char), 'MATLAB:findcllinearrange:invalidType')

% If 3 inputs, it should be vectors or arrays of numerics
verifyError(testCase, @() findcllinearrange(empty, vect10n), 'MATLAB:findcllinearrange:expectedNonempty')
verifyError(testCase, @() findcllinearrange(logi, vect10n), 'MATLAB:findcllinearrange:invalidType')
verifyError(testCase, @() findcllinearrange(char, vect10n), 'MATLAB:findcllinearrange:invalidType')
verifyError(testCase, @() findcllinearrange(vect10n, logi), 'MATLAB:findcllinearrange:invalidType')
verifyError(testCase, @() findcllinearrange(vect10n, char), 'MATLAB:findcllinearrange:invalidType')

% If 3 inputs, the arrays should have at least 3 elements
verifyError(testCase, @() findcllinearrange(vect2n, vect2n), 'MATLAB:findcllinearrange:notEnoughData')

% Inputs 2 must have same length as 1
verifyError(testCase, @() findcllinearrange(vect10n, vect11n),'MATLAB:findcllinearrange:incorrectNumrows')

end


function test_invalidPolarStructure(testCase)
% Ensure that the function throws an exception if Polar structure does not
% contain expected fields

import af_tools.*

Dummy = dummylinearpolar();
DummyAlpha = rmfield(Dummy, 'alpha');
DummyCl = rmfield(Dummy, 'cl');

verifyError(testCase, @() findcllinearrange(DummyAlpha), 'MATLAB:nonExistentField')
verifyError(testCase, @() findcllinearrange(DummyCl), 'MATLAB:nonExistentField')

end


function test_invalidPolarAlpha(testCase)
% Ensure that the function throws an exception if Polar alpha is incorrect

import af_tools.*

Dummy = dummylinearpolar();
DummyE = Dummy; DummyE.alpha = [];
DummyL = Dummy; DummyL.alpha = true;
DummyC = Dummy; DummyC.alpha = 'test';

Dummy2 = Dummy;
Dummy2.alpha = 1:2;
Dummy2.cl = 1:2;
Dummy2.cd = 1:2;

verifyError(testCase, @() findcllinearrange(DummyE), 'MATLAB:findcllinearrange:expectedNonempty')
verifyError(testCase, @() findcllinearrange(DummyL), 'MATLAB:findcllinearrange:invalidType')
verifyError(testCase, @() findcllinearrange(DummyC), 'MATLAB:findcllinearrange:invalidType')
verifyError(testCase, @() findcllinearrange(Dummy2), 'MATLAB:findcllinearrange:notEnoughData')

end


function test_invalidPolarCl(testCase)
% Ensure that the function throws an exception if Polar cl is incorrect

import af_tools.*

Dummy = dummylinearpolar();
DummyL = Dummy; DummyL.cl = true;
DummyC = Dummy; DummyC.cl = 'test';

Dummy11 = Dummy;
Dummy11.cl = 0:11;

verifyError(testCase, @() findcllinearrange(DummyL), 'MATLAB:findcllinearrange:invalidType')
verifyError(testCase, @() findcllinearrange(DummyC), 'MATLAB:findcllinearrange:invalidType')

verifyError(testCase, @() findcllinearrange(Dummy11), 'MATLAB:findcllinearrange:incorrectNumrows')

end


function test_noLinearRange(testCase)
% Ensure that the function throws an exception if no linear range was found

import af_tools.*

alpha = 0:10;
cl = rand(size(alpha));
verifyError(testCase, @() findcllinearrange(alpha, cl), 'MATLAB:findcllinearrange:noRangeFound')

end


function test_correctResults(testCase)
% Ensure that the function throws an exception if no linear range was found

import af_tools.*

alpha = deg2rad(-10:10);
cl = 2*pi*alpha;

[alphaRange, clRange, clSlope] = findcllinearrange(alpha, cl);
verifyEqual(testCase, alphaRange, [alpha(1), alpha(end)]')
verifyEqual(testCase, clRange, [cl(1), cl(end)]')
verifyEqual(testCase, clSlope, 2*pi, 'AbsTol', 1e-9)
end

