% TEST_FINDSTALL Unitary tests for the findstall function
% Note:
%   The function needs to be imported at each test, because apparently Matlab
%   does not pass it from setupOnce() or setup().
%   For some reasons, the whole package needs to be imported, not just the
%   function under testing.

% -----
% Copyright 2022 Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
% ------------------------------------------------------------------------------


%% Main test function

function tests = test_findstall

tests = functiontests(localfunctions);

end


%% Setup and teardown

function setupOnce(testCase)  % do not change function name

addpath('../.'); % Add repository to Matlab Path
addpath('./test_utils'); % Add utils to Matlab Path

% Set random number generator settings.
testCase.TestData.currentRNG = rng;

end

function teardownOnce(testCase)  % do not change function name

rmpath('../.'); % Remove repository from Matlab Path
rmpath('./test_utils'); % Remove utils from Matlab Path

% Restore the random number generator settings.
s = testCase.TestData.currentRNG;
rng(s)

end

function teardown(~)  % do not change function name

close all; % Close all figures that would have been openend

end


%% Arguments and errors tests

function test_nargin(testCase)
% Ensure that the function throws an error when called with too many arguments

import af_tools.*
dummy = (1:10)';

verifyError(testCase, @() findstall(dummy, dummy, dummy, 1), 'MATLAB:TooManyInputs')

end


function test_nargout(testCase)
% Ensure that the function throws an error too many output
% (cd_s can not be outputed if cd is not given as input)

import af_tools.*
dummy = (1:10)';

[~, ~, ~] = verifyError(testCase, @() findstall(dummy, dummy), 'MATLAB:nargoutchk:tooManyOutputs');

end


function test_invalidInputs(testCase)
% Ensure that the function throws an exception if invalid arguments (type or size).

import af_tools.*

empty = [];
scal = 0;
logi = true;
char = 'test';
vect2n = 0:1;
vect10n = 1:10;
vect11n = 1:11;

% If 1 input only, it should be a struct
verifyError(testCase, @() findstall(empty), 'MATLAB:findstall:invalidType')
verifyError(testCase, @() findstall(scal), 'MATLAB:findstall:invalidType')
verifyError(testCase, @() findstall(logi), 'MATLAB:findstall:invalidType')
verifyError(testCase, @() findstall(char), 'MATLAB:findstall:invalidType')

% If 3 inputs, it should be vectors or arrays of numerics
verifyError(testCase, @() findstall(empty, vect10n, vect10n), 'MATLAB:findstall:expectedNonempty')
verifyError(testCase, @() findstall(logi, vect10n, vect10n), 'MATLAB:findstall:invalidType')
verifyError(testCase, @() findstall(char, vect10n, vect10n), 'MATLAB:findstall:invalidType')
verifyError(testCase, @() findstall(vect10n, logi, vect10n), 'MATLAB:findstall:invalidType')
verifyError(testCase, @() findstall(vect10n, char, vect10n), 'MATLAB:findstall:invalidType')
verifyError(testCase, @() findstall(vect10n, vect10n, logi), 'MATLAB:findstall:invalidType')
verifyError(testCase, @() findstall(vect10n, vect10n, char), 'MATLAB:findstall:invalidType')

% If 3 inputs, the arrays should have at least 3 elements
verifyError(testCase, @() findstall(vect2n, vect2n, vect2n), 'MATLAB:findstall:notEnoughData')

% Inputs 2 and 3 must have same length as 1
verifyError(testCase, @() findstall(vect10n, vect11n, vect10n),'MATLAB:findstall:incorrectNumrows')
verifyError(testCase, @() findstall(vect10n, vect10n, vect11n), 'MATLAB:findstall:incorrectNumrows')

end


function test_invalidPolarStructure(testCase)
% Ensure that the function throws an exception if Polar structure does not
% contain expected fields

import af_tools.*

Dummy = dummylinearpolar();
DummyAlpha = rmfield(Dummy, 'alpha');
DummyCl = rmfield(Dummy, 'cl');
DummyCd = rmfield(Dummy, 'cd');

verifyError(testCase, @() findstall(DummyAlpha), 'MATLAB:nonExistentField')
verifyError(testCase, @() findstall(DummyCl), 'MATLAB:nonExistentField')
verifyError(testCase, @() findstall(DummyCd), 'MATLAB:nonExistentField')

end


function test_invalidPolarAlpha(testCase)
% Ensure that the function throws an exception if Polar alpha is incorrect

import af_tools.*

Dummy = dummylinearpolar();
DummyE = Dummy; DummyE.alpha = [];
DummyL = Dummy; DummyL.alpha = true;
DummyC = Dummy; DummyC.alpha = 'test';

Dummy2 = Dummy;
Dummy2.alpha = 1:2;
Dummy2.cl = 1:2;
Dummy2.cd = 1:2;

verifyError(testCase, @() findstall(DummyE), 'MATLAB:findstall:expectedNonempty')
verifyError(testCase, @() findstall(DummyL), 'MATLAB:findstall:invalidType')
verifyError(testCase, @() findstall(DummyC), 'MATLAB:findstall:invalidType')
verifyError(testCase, @() findstall(Dummy2), 'MATLAB:findstall:notEnoughData')

end


function test_invalidPolarCl(testCase)
% Ensure that the function throws an exception if Polar cl is incorrect

import af_tools.*

Dummy = dummylinearpolar();
DummyL = Dummy; DummyL.cl = true;
DummyC = Dummy; DummyC.cl = 'test';

Dummy11 = Dummy;
Dummy11.cl = 0:11;

verifyError(testCase, @() findstall(DummyL), 'MATLAB:findstall:invalidType')
verifyError(testCase, @() findstall(DummyC), 'MATLAB:findstall:invalidType')

verifyError(testCase, @() findstall(Dummy11), 'MATLAB:findstall:incorrectNumrows')

end


function test_invalidPolarCd(testCase)
% Ensure that the function throws an exception if Polar cd is incorrect

import af_tools.*

Dummy = dummylinearpolar();
DummyL = Dummy; DummyL.cd = true;
DummyC = Dummy; DummyC.cd = 'test';

Dummy11 = Dummy;
Dummy11.cd = 0:11;

verifyError(testCase, @() findstall(DummyL), 'MATLAB:findstall:invalidType')
verifyError(testCase, @() findstall(DummyC), 'MATLAB:findstall:invalidType')

verifyError(testCase, @() findstall(Dummy11), 'MATLAB:findstall:incorrectNumrows')

end


function test_noStall(testCase)
% Ensure that the function throws an exception if no stall was found

import af_tools.*

Polar = dummylinearpolar();

verifyError(testCase, @() findstall(Polar), 'MATLAB:findstall:noStallFound')

end


function test_correctResults(testCase)
% Check if function works as intended on basic cases

import af_tools.*

Polar = dummysinepolar(); % Stall at alpha_s = pi/2, cl_s=1; cd_s = 0;

[alpha_s, cl_s, cd_s] = findstall(Polar);
[alpha_s2, cl_s2, cd_s2] = findstall(Polar.alpha, Polar.cl, Polar.cd);

verifyEqual(testCase, alpha_s, pi/2, 'AbsTol', 1e-9)
verifyEqual(testCase, cl_s, 1, 'AbsTol', 1e-9)
verifyEqual(testCase, cd_s, 0, 'AbsTol', 1e-9)
verifyEqual(testCase, alpha_s2, pi/2, 'AbsTol', 1e-9)
verifyEqual(testCase, cl_s2, 1, 'AbsTol', 1e-9)
verifyEqual(testCase, cd_s2, 0, 'AbsTol', 1e-9)
end