% TEST_findzerolift Unitary tests for the findzerolift function
% Note:
%   The function needs to be imported at each test, because apparently Matlab
%   does not pass it from setupOnce() or setup().
%   For some reasons, the whole package needs to be imported, not just the
%   function under testing.

% -----
% Copyright 2022 Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
% ------------------------------------------------------------------------------


%% Main test function

function tests = test_findzerolift

tests = functiontests(localfunctions);

end


%% Setup and teardown

function setupOnce(testCase)  % do not change function name

addpath('../.'); % Add repository to Matlab Path
addpath('./test_utils'); % Add utils to Matlab Path

% Set random number generator settings.
testCase.TestData.currentRNG = rng;

end

function teardownOnce(testCase)  % do not change function name

rmpath('../.'); % Remove repository from Matlab Path
rmpath('./test_utils'); % Remove utils from Matlab Path

% Restore the random number generator settings.
s = testCase.TestData.currentRNG;
rng(s)

end

function teardown(~)  % do not change function name

close all; % Close all figures that would have been openend

end


%% Arguments and errors tests

function test_nargin(testCase)
% Ensure that the function throws an error when called with too many arguments

import af_tools.*
dummy = (1:10)';

verifyError(testCase, @() findzerolift(dummy, dummy, dummy, 1), 'MATLAB:TooManyInputs')

end


function test_nargout(testCase)
% Ensure that the function throws an error too many output
% (cd_s can not be outputed if cd is not given as input)

import af_tools.*
dummy = (1:10)';

[~, ~] = verifyError(testCase, @() findzerolift(dummy, dummy), 'MATLAB:nargoutchk:tooManyOutputs');

end


function test_invalidInputs(testCase)
% Ensure that the function throws an exception if invalid arguments (type or size).

import af_tools.*

empty = [];
scal = 0;
logi = true;
char = 'test';
vect2n = 0:1;
vect10n = 1:10;
vect11n = 1:11;

% If 1 input only, it should be a struct
verifyError(testCase, @() findzerolift(empty), 'MATLAB:findzerolift:invalidType')
verifyError(testCase, @() findzerolift(scal), 'MATLAB:findzerolift:invalidType')
verifyError(testCase, @() findzerolift(logi), 'MATLAB:findzerolift:invalidType')
verifyError(testCase, @() findzerolift(char), 'MATLAB:findzerolift:invalidType')

% If 3 inputs, it should be vectors or arrays of numerics
verifyError(testCase, @() findzerolift(empty, vect10n, vect10n), 'MATLAB:findzerolift:expectedNonempty')
verifyError(testCase, @() findzerolift(logi, vect10n, vect10n), 'MATLAB:findzerolift:invalidType')
verifyError(testCase, @() findzerolift(char, vect10n, vect10n), 'MATLAB:findzerolift:invalidType')
verifyError(testCase, @() findzerolift(vect10n, logi, vect10n), 'MATLAB:findzerolift:invalidType')
verifyError(testCase, @() findzerolift(vect10n, char, vect10n), 'MATLAB:findzerolift:invalidType')
verifyError(testCase, @() findzerolift(vect10n, vect10n, logi), 'MATLAB:findzerolift:invalidType')
verifyError(testCase, @() findzerolift(vect10n, vect10n, char), 'MATLAB:findzerolift:invalidType')

% If 3 inputs, the arrays should have at least 3 elements
verifyError(testCase, @() findzerolift(vect2n, vect2n, vect2n), 'MATLAB:findzerolift:notEnoughData')

% Inputs 2 and 3 must have same length as 1
verifyError(testCase, @() findzerolift(vect10n, vect11n, vect10n),'MATLAB:findzerolift:incorrectNumrows')
verifyError(testCase, @() findzerolift(vect10n, vect10n, vect11n), 'MATLAB:findzerolift:incorrectNumrows')

end


function test_invalidPolarStructure(testCase)
% Ensure that the function throws an exception if Polar structure does not
% contain expected fields

import af_tools.*

Dummy = dummylinearpolar();
DummyAlpha = rmfield(Dummy, 'alpha');
DummyCl = rmfield(Dummy, 'cl');
DummyCd = rmfield(Dummy, 'cd');

verifyError(testCase, @() findzerolift(DummyAlpha), 'MATLAB:nonExistentField')
verifyError(testCase, @() findzerolift(DummyCl), 'MATLAB:nonExistentField')
verifyError(testCase, @() findzerolift(DummyCd), 'MATLAB:nonExistentField')

end


function test_invalidPolarAlpha(testCase)
% Ensure that the function throws an exception if Polar alpha is incorrect

import af_tools.*

Dummy = dummylinearpolar();
DummyE = Dummy; DummyE.alpha = [];
DummyL = Dummy; DummyL.alpha = true;
DummyC = Dummy; DummyC.alpha = 'test';

Dummy2 = Dummy;
Dummy2.alpha = 1:2;
Dummy2.cl = 1:2;
Dummy2.cd = 1:2;

verifyError(testCase, @() findzerolift(DummyE), 'MATLAB:findzerolift:expectedNonempty')
verifyError(testCase, @() findzerolift(DummyL), 'MATLAB:findzerolift:invalidType')
verifyError(testCase, @() findzerolift(DummyC), 'MATLAB:findzerolift:invalidType')
verifyError(testCase, @() findzerolift(Dummy2), 'MATLAB:findzerolift:notEnoughData')

end


function test_invalidPolarCl(testCase)
% Ensure that the function throws an exception if Polar cl is incorrect

import af_tools.*

Dummy = dummylinearpolar();
DummyL = Dummy; DummyL.cl = true;
DummyC = Dummy; DummyC.cl = 'test';

Dummy11 = Dummy;
Dummy11.cl = 0:11;

verifyError(testCase, @() findzerolift(DummyL), 'MATLAB:findzerolift:invalidType')
verifyError(testCase, @() findzerolift(DummyC), 'MATLAB:findzerolift:invalidType')

verifyError(testCase, @() findzerolift(Dummy11), 'MATLAB:findzerolift:incorrectNumrows')

end


function test_invalidPolarCd(testCase)
% Ensure that the function throws an exception if Polar cd is incorrect

import af_tools.*

Dummy = dummylinearpolar();
DummyL = Dummy; DummyL.cd = true;
DummyC = Dummy; DummyC.cd = 'test';

Dummy11 = Dummy;
Dummy11.cd = 0:11;

verifyError(testCase, @() findzerolift(DummyL), 'MATLAB:findzerolift:invalidType')
verifyError(testCase, @() findzerolift(DummyC), 'MATLAB:findzerolift:invalidType')

verifyError(testCase, @() findzerolift(Dummy11), 'MATLAB:findzerolift:incorrectNumrows')

end


function test_noPositiveCl(testCase)
% Ensure that the function throws an exception if no positive Cl was found

import af_tools.*

Polar = dummylinearpolar();
Polar.cl = -(1:11)';
verifyError(testCase, @() findzerolift(Polar), 'MATLAB:findzerolift:noPositiveCl')

end


function test_noNegativeCl(testCase)
% Ensure that the function throws an exception if no negative Cl was found

import af_tools.*

Polar = dummylinearpolar();
Polar.cl = (1:11)';
verifyError(testCase, @() findzerolift(Polar), 'MATLAB:findzerolift:noNegativeCl')

end


function test_correctResults(testCase)
% Check if function works as intended on basic cases

import af_tools.*

Polar = dummylinearpolar(); % Zero lift angle at alpha = 0, cd = 0

[alpha_zeroL, cd_zeroL] = findzerolift(Polar);
[alpha_zeroL2, cd_zeroL2] = findzerolift(Polar.alpha, Polar.cl, Polar.cd);

verifyEqual(testCase, alpha_zeroL, 0)
verifyEqual(testCase, cd_zeroL, 0)
verifyEqual(testCase, alpha_zeroL2, 0)
verifyEqual(testCase, cd_zeroL2, 0)
end