% TEST_NACAAIRFOIL Unitary tests for the nacaairfoil function
% Note:
%   The function needs to be imported at each test, because apparently Matlab
%   does not pass it from setupOnce() or setup().
%   For some reasons, the whole package needs to be imported, not just the
%   function under testing.

% -----
% Copyright 2022 Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
% ------------------------------------------------------------------------------


%% Main test function

function tests = test_nacaairfoil

tests = functiontests(localfunctions);

end


%% Setup and teardown

function setupOnce(testCase)  % do not change function name

addpath('../.'); % Add repository to Matlab Path
addpath('./test_utils'); % Add utils to Matlab Path

% Set random number generator settings.
testCase.TestData.currentRNG = rng;

end

function teardownOnce(testCase)  % do not change function name

rmpath('../.'); % Remove repository from Matlab Path
rmpath('./test_utils'); % Remove utils from Matlab Path

% Restore the random number generator settings.
s = testCase.TestData.currentRNG;
rng(s)

end

function teardown(~)  % do not change function name

close all; % Close all figures that would have been openend

end


%% Arguments and errors tests

function test_nargin(testCase)
% Ensure that the function throws an error when called with too many arguments

import af_tools.*

verifyError(testCase, @() nacaairfoil('0012', 100, 'halfcosine', true, 1), 'MATLAB:TooManyInputs')

end


function test_invalidInputs(testCase)
% Ensure that the function throws an exception if invalid arguments (type or size).

import af_tools.*

empty = [];
scal = 0;
logi = true;
vect = 1:2;
char = 'test';
onePoint = 1;

% Digits
verifyError(testCase, @() nacaairfoil(empty, 100, 'halfcosine'), 'MATLAB:nacaairfoil:invalidType')
verifyError(testCase, @() nacaairfoil(scal, 100, 'halfcosine'), 'MATLAB:nacaairfoil:invalidType')
verifyError(testCase, @() nacaairfoil(logi, 100, 'halfcosine'), 'MATLAB:nacaairfoil:invalidType')

% nPoints
verifyError(testCase, @() nacaairfoil('0012', empty, 'halfcosine'), 'MATLAB:nacaairfoil:expectedScalar')
verifyError(testCase, @() nacaairfoil('0012', vect, 'halfcosine'), 'MATLAB:nacaairfoil:expectedScalar')
verifyError(testCase, @() nacaairfoil('0012', char, 'halfcosine'), 'MATLAB:nacaairfoil:invalidType')
verifyError(testCase, @() nacaairfoil('0012', logi, 'halfcosine'), 'MATLAB:nacaairfoil:invalidType')
verifyError(testCase, @() nacaairfoil('0012', onePoint, 'halfcosine'), 'MATLAB:nacaairfoil:notGreaterEqual')

% Spacing
verifyError(testCase, @() nacaairfoil('0012', 100, empty), 'MATLAB:nacaairfoil:invalidType')
verifyError(testCase, @() nacaairfoil('0012', 100, logi), 'MATLAB:nacaairfoil:invalidType')
verifyError(testCase, @() nacaairfoil('0012', 100, scal), 'MATLAB:nacaairfoil:invalidType')

% finteTrail
verifyError(testCase, @() nacaairfoil('0012', 100, 'halfcosine', empty), 'MATLAB:nacaairfoil:invalidType')
verifyError(testCase, @() nacaairfoil('0012', 100, 'halfcosine', scal), 'MATLAB:nacaairfoil:invalidType')
verifyError(testCase, @() nacaairfoil('0012', 100, 'halfcosine', char), 'MATLAB:nacaairfoil:invalidType')

end


function test_wrongNumberOfDigitss(testCase)
% Ensure that the function throws an exception if wrong number of digits.

import af_tools.*

naca3 = '012';
naca6 = '012345';

verifyError(testCase, @() nacaairfoil(naca3), 'MATLAB:nacaairfoil:wrongNumberOfDigits')
verifyError(testCase, @() nacaairfoil(naca6), 'MATLAB:nacaairfoil:wrongNumberOfDigits')

end


function test_invalidDigits(testCase)
% Ensure that the function throws an exception if invalid digits.

import af_tools.*

nonDigits = 'abcd';
wrong5 = '24215'; % 3rd digit of a 5 series must be 1 or 0

verifyError(testCase, @() nacaairfoil(nonDigits), 'MATLAB:nacacamber:invalidDigits')
verifyError(testCase, @() nacaairfoil(wrong5), 'MATLAB:nacacamber:invalid5Digits')

end


function test_invalidSpacing(testCase)
% Ensure that the function throws an exception if invalid spacing.

import af_tools.*

wrongSpacing = 'testspacing';
verifyError(testCase, @() nacaairfoil('0012', 100, wrongSpacing), 'MATLAB:nacaairfoil:unrecognizedStringChoice')

end


function test_numberPoints(testCase)
% Test if output vector contain the correct amount of points
% Note:
%   nacaairfoil will always output an even number of points, the upper and
%   lower surfaces will have the same number of points and the leading
%   edge is counted twice while the trailing edge is counted once.

import af_tools.*

evenNPoints = 240;
oddNPoints = 241;

[xc4even, yc4even] = nacaairfoil('0012', evenNPoints);
[xc5even, yc5even] = nacaairfoil('24012', evenNPoints);
[xc4odd, yc4odd] = nacaairfoil('0012', oddNPoints);
[xc5odd, yc5odd] = nacaairfoil('24012', oddNPoints);

verifyEqual(testCase, length(xc4even), evenNPoints+1)
verifyEqual(testCase, length(yc4even), evenNPoints+1)
verifyEqual(testCase, length(xc5even), evenNPoints+1)
verifyEqual(testCase, length(yc5even), evenNPoints+1)
verifyEqual(testCase, length(xc4odd), oddNPoints)
verifyEqual(testCase, length(yc4odd), oddNPoints)
verifyEqual(testCase, length(xc5odd), oddNPoints)
verifyEqual(testCase, length(yc5odd), oddNPoints)

end


function test_spacing(testCase)
% Test if xc follows the correct spacing rule

import af_tools.*

nPoints = 643;

linear = linspace(0, 1, ceil((nPoints+1)/2));
betahalfcos = linspace(0, pi, ceil((nPoints+1)/2));
halfcos = (1-cos(betahalfcos))/2;
betacos = linspace(0, pi/2, ceil((nPoints+1)/2));
cosine = 1-cos(betacos);

[xc_lin, ~] = nacaairfoil('0012', nPoints, 'linear');
[xc_cos, ~] = nacaairfoil('0012', nPoints, 'cosine');
[xc_half, ~] = nacaairfoil('0012', nPoints, 'halfcosine');

verifyEqual(testCase, xc_lin(1:(end+1)/2), linear)
verifyEqual(testCase, xc_cos(1:(end+1)/2), cosine)
verifyEqual(testCase, xc_half(1:(end+1)/2), halfcos)

end


function test_defaults(testCase)
% Test if defaults are properly set

import af_tools.*

DEF_NPOINTS = 100;
DEF_SPACING = 'halfcosine';
DEF_FINITE_TE = false;

[xc, yc] = nacaairfoil('0012');
[xc_def, yc_def] = nacaairfoil('0012', DEF_NPOINTS, DEF_SPACING, DEF_FINITE_TE);

verifyEqual(testCase, xc, xc_def)
verifyEqual(testCase, yc, yc_def)

end