% TEST_NACACAMBER Unitary tests for the nacacamber function
% Note:
%   The function needs to be imported at each test, because apparently Matlab
%   does not pass it from setupOnce() or setup().
%   For some reasons, the whole package needs to be imported, not just the
%   function under testing.

% -----
% Copyright 2022 Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
% ------------------------------------------------------------------------------


%% Main test function

function tests = test_nacacamber

tests = functiontests(localfunctions);

end


%% Setup and teardown

function setupOnce(testCase)  % do not change function name

addpath('../.'); % Add repository to Matlab Path
addpath('./test_utils'); % Add utils to Matlab Path

% Set random number generator settings.
testCase.TestData.currentRNG = rng;

end

function teardownOnce(testCase)  % do not change function name

rmpath('../.'); % Remove repository from Matlab Path
rmpath('./test_utils'); % Remove utils from Matlab Path

% Restore the random number generator settings.
s = testCase.TestData.currentRNG;
rng(s)

end

function teardown(~)  % do not change function name

close all; % Close all figures that would have been openend

end


%% Arguments and errors tests

function test_nargin(testCase)
% Ensure that the function throws an error when called with too many arguments

import af_tools.*

verifyError(testCase, @() nacacamber('0012', 100, 'halfcosine', 1), 'MATLAB:TooManyInputs')

end


function test_invalidInputs(testCase)
% Ensure that the function throws an exception if invalid arguments (type or size).

import af_tools.*

empty = [];
scal = 0;
logi = true;
vect = 1:2;
char = 'test';
onePoint = 1;

% Digits
verifyError(testCase, @() nacacamber(empty, 100, 'halfcosine'), 'MATLAB:nacacamber:invalidType')
verifyError(testCase, @() nacacamber(scal, 100, 'halfcosine'), 'MATLAB:nacacamber:invalidType')
verifyError(testCase, @() nacacamber(logi, 100, 'halfcosine'), 'MATLAB:nacacamber:invalidType')

% nPoints
verifyError(testCase, @() nacacamber('0012', empty, 'halfcosine'), 'MATLAB:nacacamber:expectedScalar')
verifyError(testCase, @() nacacamber('0012', vect, 'halfcosine'), 'MATLAB:nacacamber:expectedScalar')
verifyError(testCase, @() nacacamber('0012', char, 'halfcosine'), 'MATLAB:nacacamber:invalidType')
verifyError(testCase, @() nacacamber('0012', logi, 'halfcosine'), 'MATLAB:nacacamber:invalidType')
verifyError(testCase, @() nacacamber('0012', onePoint, 'halfcosine'), 'MATLAB:nacacamber:notGreaterEqual')

% Spacing
verifyError(testCase, @() nacacamber('0012', 100, empty), 'MATLAB:nacacamber:invalidType')
verifyError(testCase, @() nacacamber('0012', 100, logi), 'MATLAB:nacacamber:invalidType')
verifyError(testCase, @() nacacamber('0012', 100, scal), 'MATLAB:nacacamber:invalidType')

end


function test_warningLowNPoints(testCase)
% Ensure that the function throws a warning for low number of points.

import af_tools.*

lowNPoints = 5;

verifyWarning(testCase, @() nacacamber('0012', lowNPoints, 'halfcosine'), 'MATLAB:nacacamber:lowNPoints')


end


function test_wrongNumberOfDigitss(testCase)
% Ensure that the function throws an exception if wrong number of digits.

import af_tools.*

naca3 = '012';
naca6 = '012345';

verifyError(testCase, @() nacacamber(naca3), 'MATLAB:nacacamber:wrongNumberOfDigits')
verifyError(testCase, @() nacacamber(naca6), 'MATLAB:nacacamber:wrongNumberOfDigits')

end


function test_invalidDigits(testCase)
% Ensure that the function throws an exception if invalid digits.

import af_tools.*

nonDigits = 'abcd';
wrong5 = '24215'; % 3rd digit of a 5 series must be 1 or 0

verifyError(testCase, @() nacacamber(nonDigits), 'MATLAB:nacacamber:invalidDigits')
verifyError(testCase, @() nacacamber(wrong5), 'MATLAB:nacacamber:invalid5Digits')

end


function test_invalidSpacing(testCase)
% Ensure that the function throws an exception if invalid spacing.

import af_tools.*

wrongSpacing = 'testspacing';
verifyError(testCase, @() nacacamber('0012', 100, wrongSpacing), 'MATLAB:nacacamber:unrecognizedStringChoice')

end


function test_flatCamber(testCase)
% Test if output of symmetric airfoil is all zeros yc

import af_tools.*

[~, yc4] = nacacamber('0012');
[~, yc5] = nacacamber('20012');
expected_yc = zeros(1,100);

verifyEqual(testCase, yc4, expected_yc)
verifyEqual(testCase, yc5, expected_yc)

end


function test_numberPoints(testCase)
% Test if output vector contain the correct amount of points

import af_tools.*

nPoints = 120;

[xc4, yc4] = nacacamber('0012', nPoints);
[xc5, yc5] = nacacamber('24012', nPoints);

verifyEqual(testCase, length(xc4), nPoints)
verifyEqual(testCase, length(yc4), nPoints)
verifyEqual(testCase, length(xc5), nPoints)
verifyEqual(testCase, length(yc5), nPoints)

end


function test_spacing(testCase)
% Test if xc follows the correct spacing rule

import af_tools.*

nPoints = 222;

linear = linspace(0, 1, nPoints);
betahalfcos = linspace(0, pi, nPoints);
halfcos = (1-cos(betahalfcos))/2;
betacos = linspace(0, pi/2, nPoints);
cosine = 1-cos(betacos);

[xc_lin, ~] = nacacamber('0012', nPoints, 'linear');
[xc_cos, ~] = nacacamber('0012', nPoints, 'cosine');
[xc_half, ~] = nacacamber('0012', nPoints, 'halfcosine');

verifyEqual(testCase, xc_lin, linear)
verifyEqual(testCase, xc_cos, cosine)
verifyEqual(testCase, xc_half, halfcos)

end


function test_defaults(testCase)
% Test if defaults are properly set

import af_tools.*

DEF_NPOINTS = 100;
DEF_SPACING = 'halfcosine';

[xc, yc] = nacacamber('0012');
[xc_def, yc_def] = nacacamber('0012',DEF_NPOINTS, DEF_SPACING);

verifyEqual(testCase, xc, xc_def)
verifyEqual(testCase, yc, yc_def)

end


function test_maxCamber(testCase)
% Test if maxCamber is where it should be

import af_tools.*

% Pick a random NACA
naca4 = '6409';
naca5 = '24012';

% Get expected camber values and position
maxCambVal4 = str2double(naca4(1))/100;
maxCambPos4 = str2double(naca4(2))/10;
maxCambPos5 = str2double(naca5(2))/20;

% Calculate camberline
[xc4, yc4] = nacacamber(naca4, 100000, 'linear');
[xc5, yc5] = nacacamber(naca5, 100000, 'linear');

% Get max camber value and position from camberline coordinates
[maxVal4, maxPos4] = max(yc4);
[~, maxPos5] = max(yc5);

% Naca 4
verifyEqual(testCase, maxVal4, maxCambVal4, 'RelTol', 1e-3)
verifyEqual(testCase, xc4(maxPos4), maxCambPos4, 'RelTol', 1e-3)
% Naca 5
verifyEqual(testCase, xc5(maxPos5), maxCambPos5, 'RelTol', 1e-3)

end