% TEST_PLOTPOLARS Unitary tests for the plotpolars function
% Note:
%   The function needs to be imported at each test, because apparently Matlab
%   does not pass it from setupOnce() or setup().
%   For some reasons, the whole package needs to be imported, not just the
%   function under testing.

% -----
% Copyright 2022 Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
% ------------------------------------------------------------------------------


%% Main test function

function tests = test_plotpolars

tests = functiontests(localfunctions);

end


%% Setup and teardown

function setupOnce(testCase)  % do not change function name

addpath('../.'); % Add repository to Matlab Path
addpath('./test_utils'); % Add utils to Matlab Path

% Set random number generator settings.
testCase.TestData.currentRNG = rng;

end

function teardownOnce(testCase)  % do not change function name

rmpath('../.'); % Remove repository from Matlab Path
rmpath('./test_utils'); % Remove utils from Matlab Path

% Restore the random number generator settings.
s = testCase.TestData.currentRNG;
rng(s)

end

function teardown(~)  % do not change function name

close all; % Close all figures that would have been openend

end


%% Arguments and errors tests

function test_nargin(testCase)
% Ensure that the function throws an error when called with too many arguments

import af_tools.*
dummy = (1:10)';

verifyError(testCase, @() plotpolars(dummy, dummy, dummy, dummy, 1), 'MATLAB:TooManyInputs')

end


function test_invalidInputs(testCase)
% Ensure that the function throws an exception if invalid arguments (type or size).

import af_tools.*

empty = [];
scal = 0;
logi = true;
char = 'test';
vect10n = 1:10;
vect11n = 1:11;

% If 1 input only, it should be a struct
verifyError(testCase, @() plotpolars(empty), 'MATLAB:plotpolars:invalidType')
verifyError(testCase, @() plotpolars(scal), 'MATLAB:plotpolars:invalidType')
verifyError(testCase, @() plotpolars(logi), 'MATLAB:plotpolars:invalidType')
verifyError(testCase, @() plotpolars(char), 'MATLAB:plotpolars:invalidType')

% If 3 inputs, it should be vectors or arrays of numerics
verifyError(testCase, @() plotpolars(empty, vect10n, vect10n), 'MATLAB:plotpolars:expectedNonempty')
verifyError(testCase, @() plotpolars(logi, vect10n, vect10n), 'MATLAB:plotpolars:invalidType')
verifyError(testCase, @() plotpolars(char, vect10n, vect10n), 'MATLAB:plotpolars:invalidType')
verifyError(testCase, @() plotpolars(vect10n, logi, vect10n), 'MATLAB:plotpolars:invalidType')
verifyError(testCase, @() plotpolars(vect10n, char, vect10n), 'MATLAB:plotpolars:invalidType')
verifyError(testCase, @() plotpolars(vect10n, vect10n, logi), 'MATLAB:plotpolars:invalidType')
verifyError(testCase, @() plotpolars(vect10n, vect10n, char), 'MATLAB:plotpolars:invalidType')
verifyError(testCase, @() plotpolars(vect10n, vect10n, vect10n, logi), 'MATLAB:plotpolars:invalidType')
verifyError(testCase, @() plotpolars(vect10n, vect10n, vect10n, char), 'MATLAB:plotpolars:invalidType')

% Inputs 2 and 3 must have same length as 1
verifyError(testCase, @() plotpolars(vect10n, vect11n, vect10n),'MATLAB:plotpolars:incorrectNumrows')
verifyError(testCase, @() plotpolars(vect10n, vect10n, vect11n), 'MATLAB:plotpolars:incorrectNumrows')
verifyError(testCase, @() plotpolars(vect10n, vect10n, vect10n, vect11n), 'MATLAB:plotpolars:incorrectNumrows')

end


function test_invalidPolarStructure(testCase)
% Ensure that the function throws an exception if Polar structure does not
% contain expected fields

import af_tools.*

Dummy = dummylinearpolar();
DummyAlpha = rmfield(Dummy, 'alpha');
DummyCl = rmfield(Dummy, 'cl');
DummyCd = rmfield(Dummy, 'cd');
DummyCm = rmfield(Dummy, 'cm');

verifyError(testCase, @() plotpolars(DummyAlpha), 'MATLAB:nonExistentField')
verifyError(testCase, @() plotpolars(DummyCl), 'MATLAB:nonExistentField')
verifyError(testCase, @() plotpolars(DummyCd), 'MATLAB:nonExistentField')
verifyError(testCase, @() plotpolars(DummyCm), 'MATLAB:nonExistentField')

end


function test_invalidPolarAlpha(testCase)
% Ensure that the function throws an exception if Polar alpha is incorrect

import af_tools.*

Dummy = dummylinearpolar();
DummyE = Dummy; DummyE.alpha = [];
DummyL = Dummy; DummyL.alpha = true;
DummyC = Dummy; DummyC.alpha = 'test';

Dummy2 = Dummy;
Dummy2.alpha = 1:2;
Dummy2.cl = 1:2;
Dummy2.cd = 1:2;
Dummy2.cm = 1:2;

verifyError(testCase, @() plotpolars(DummyE), 'MATLAB:plotpolars:expectedNonempty')
verifyError(testCase, @() plotpolars(DummyL), 'MATLAB:plotpolars:invalidType')
verifyError(testCase, @() plotpolars(DummyC), 'MATLAB:plotpolars:invalidType')

end


function test_invalidPolarCl(testCase)
% Ensure that the function throws an exception if Polar cl is incorrect

import af_tools.*

Dummy = dummylinearpolar();
DummyL = Dummy; DummyL.cl = true;
DummyC = Dummy; DummyC.cl = 'test';

Dummy11 = Dummy;
Dummy11.cl = 0:11;

verifyError(testCase, @() plotpolars(DummyL), 'MATLAB:plotpolars:invalidType')
verifyError(testCase, @() plotpolars(DummyC), 'MATLAB:plotpolars:invalidType')

verifyError(testCase, @() plotpolars(Dummy11), 'MATLAB:plotpolars:incorrectNumrows')

end


function test_invalidPolarCd(testCase)
% Ensure that the function throws an exception if Polar cd is incorrect

import af_tools.*

Dummy = dummylinearpolar();
DummyL = Dummy; DummyL.cd = true;
DummyC = Dummy; DummyC.cd = 'test';

Dummy11 = Dummy;
Dummy11.cd = 0:11;

verifyError(testCase, @() plotpolars(DummyL), 'MATLAB:plotpolars:invalidType')
verifyError(testCase, @() plotpolars(DummyC), 'MATLAB:plotpolars:invalidType')

verifyError(testCase, @() plotpolars(Dummy11), 'MATLAB:plotpolars:incorrectNumrows')

end


function test_invalidPolarCm(testCase)
% Ensure that the function throws an exception if Polar cm is incorrect

import af_tools.*

Dummy = dummylinearpolar();
DummyL = Dummy; DummyL.cm = true;
DummyC = Dummy; DummyC.cm = 'test';

Dummy11 = Dummy;
Dummy11.cm = 0:11;

verifyError(testCase, @() plotpolars(DummyL), 'MATLAB:plotpolars:invalidType')
verifyError(testCase, @() plotpolars(DummyC), 'MATLAB:plotpolars:invalidType')

verifyError(testCase, @() plotpolars(Dummy11), 'MATLAB:plotpolars:incorrectNumrows')

end

